import lldb
from lldbsuite.test.decorators import *
import lldbsuite.test.lldbtest as lldbtest
import lldbsuite.test.lldbutil as lldbutil
import unittest2
import re

class TestSwiftReflectionOnly(lldbtest.TestBase):
    NO_DEBUG_INFO_TESTCASE = True

    mydir = lldbtest.TestBase.compute_mydir(__file__)

    @swiftTest
    def test(self):
        """Test debugging a program without swiftmodules is functional"""
        self.build()

        target, process, thread, bkpt = lldbutil.run_to_source_breakpoint(
            self, 'Set breakpoint here', lldb.SBFileSpec('main.swift'),
            extra_images=['dynamic_lib'])
        log = self.getBuildArtifact('types.log')
        self.expect('log enable lldb types -f ' + log)

        check_var = lldbutil.check_variable
        frame = thread.frames[0]
        var_self = frame.FindVariable("self")
        var_self_x = var_self.GetChildMemberWithName("x")
        check_var(self, var_self_x, value="42")

        check_var(self, frame.FindVariable("number"), value="1")

        array = frame.FindVariable("array")
        check_var(self, array, num_children=3)
        check_var(self, array.GetChildAtIndex(0), value="1")
        check_var(self, array.GetChildAtIndex(1), value="2")
        check_var(self, array.GetChildAtIndex(2), value="3")

        check_var(self, frame.FindVariable("string"), summary='"hello"')

        tup = frame.FindVariable("tuple")
        check_var(self, tup, num_children=2)
        check_var(self, tup.GetChildAtIndex(0), value="0")
        check_var(self, tup.GetChildAtIndex(1), value="1")

        strct = frame.FindVariable("strct")
        check_var(self, strct, num_children=5)
        check_var(self, strct.GetChildMemberWithName("pub"), value="1")
        check_var(self, strct.GetChildMemberWithName("priv"), value="2")
        check_var(self, strct.GetChildMemberWithName("filepriv"), value="3")
        s_priv = strct.GetChildMemberWithName("s_priv")
        check_var(self, s_priv, num_children=1)
        check_var(self, s_priv.GetChildMemberWithName("i"), value="2")
        s_filepriv = strct.GetChildMemberWithName("s_filepriv")
        check_var(self, s_filepriv, num_children=1)
        check_var(self, s_filepriv.GetChildMemberWithName("i"), value="3")

        check_var(self, frame.FindVariable("generic"), use_dynamic=True, value="42")

        gtup = frame.FindVariable("generic_tuple", lldb.eNoDynamicValues)
        check_var(self, gtup, num_children=2)
        check_var(self, gtup.GetChildAtIndex(0), use_dynamic=True, value="42")
        check_var(self, gtup.GetChildAtIndex(1), use_dynamic=True, value="42")

        check_var(self, frame.FindVariable("word"), value="0")
        check_var(self, frame.FindVariable("enum1"), value="second")
        enum2 = frame.FindVariable("enum2")
        check_var(self, enum2, value="with")
        check_var(self, enum2, num_children=1)
        check_var(self, enum2.GetChildAtIndex(0).GetChildAtIndex(0), value="42")

        # Scan through the types log.
        import io
        logfile = io.open(log, "r", encoding='utf-8')
        found_ref_exe = 0
        found_ref_lib = 0
        found_ast_exe = 0
        found_ast_lib = 0
        for line in logfile:
            if 'SwiftASTContextForExpressions::RegisterSectionModules("a.out");' in line:
                if not 'retrieved 0 AST Data blobs' in line:
                    found_ast_exe += 1
            elif 'SwiftASTContextForExpressions::RegisterSectionModules("dyld")' in line:
                if not 'retrieved 0 AST Data blobs' in line:
                    found_ast_lib += 1
            elif re.search(r'Adding reflection metadata in .*a\.out', line):
                found_ref_exe += 1
            elif re.search(r'Adding reflection metadata in .*dynamic_lib', line):
                found_ref_lib += 1
        self.assertEqual(found_ref_exe, 1)
        self.assertEqual(found_ref_lib, 1)
        self.assertEqual(found_ast_exe, 0)
        self.assertEqual(found_ast_lib, 0)



# TestSwiftGenericSelf.py
#
# This source file is part of the Swift.org open source project
#
# Copyright (c) 2014 - 2016 Apple Inc. and the Swift project authors
# Licensed under Apache License v2.0 with Runtime Library Exception
#
# See https://swift.org/LICENSE.txt for license information
# See https://swift.org/CONTRIBUTORS.txt for the list of Swift project authors
#
# ------------------------------------------------------------------------------
import lldbsuite.test.lldbinline as lldbinline
from lldbsuite.test.decorators import *

lldbinline.MakeInlineTest(__file__, globals(), decorators=[swiftTest])



from lldbsuite.test import lldbinline
from lldbsuite.test import decorators

# https://bugs.llvm.org/show_bug.cgi?id=35920
# This test stresses expression evaluation support for template functions.
# Currently the support is rudimentary, and running this test causes assertion
# failures in clang. This test cannot be XFAIL'ed because the test harness
# treats assertion failures as unexpected events. For now, the test must be
# skipped.
lldbinline.MakeInlineTest(__file__, globals(), [decorators.skipIf])



# TestExpressionScopes.py
#
# This source file is part of the Swift.org open source project
#
# Copyright (c) 2014 - 2016 Apple Inc. and the Swift project authors
# Licensed under Apache License v2.0 with Runtime Library Exception
#
# See https://swift.org/LICENSE.txt for license information
# See https://swift.org/CONTRIBUTORS.txt for the list of Swift project authors
#
# ------------------------------------------------------------------------------
"""
Tests scoped variables with swift expressions
"""
import lldb
from lldbsuite.test.lldbtest import *
from lldbsuite.test.decorators import *
import lldbsuite.test.lldbutil as lldbutil
import os
import unittest2


class TestSwiftExpressionScopes(TestBase):
    @swiftTest
    def test_expression_scopes(self):
        """Tests that swift expressions resolve scoped variables correctly"""
        self.build()
        self.do_test()

    def setUp(self):
        TestBase.setUp(self)
        self.main_source = "main.swift"
        self.main_source_spec = lldb.SBFileSpec(self.main_source)

    def continue_to_bkpt(self, process, bkpt):
        threads = lldbutil.continue_to_breakpoint(process, bkpt)
        self.assertTrue(len(threads) == 1)

    def continue_by_pattern(self, pattern):
        bkpt = self.target.BreakpointCreateBySourceRegex(
            pattern, self.main_source_spec)
        self.assertTrue(bkpt.GetNumLocations() > 0, VALID_BREAKPOINT)
        self.continue_to_bkpt(self.process, bkpt)
        self.target.BreakpointDelete(bkpt.GetID())

    def do_test(self):
        """Tests that swift expressions resolve scoped variables correctly"""
        exe_name = "a.out"
        exe = self.getBuildArtifact(exe_name)

        # Create the target
        target = self.dbg.CreateTarget(exe)
        self.target = target
        self.assertTrue(target, VALID_TARGET)

        # Set the breakpoints
        shadow_a_bkpt = target.BreakpointCreateBySourceRegex(
            'Shadowed in A', self.main_source_spec)
        self.assertTrue(shadow_a_bkpt.GetNumLocations() > 0, VALID_BREAKPOINT)

        init_bkpt = target.BreakpointCreateBySourceRegex(
            'In init.', self.main_source_spec)
        self.assertTrue(init_bkpt.GetNumLocations() > 0, VALID_BREAKPOINT)

        # Launch the process, and do not stop at the entry point.
        process = target.LaunchSimple(None, None, os.getcwd())
        self.process = process

        self.assertTrue(process, PROCESS_IS_VALID)

        # Frame #0 should be at our breakpoint.
        threads = lldbutil.get_threads_stopped_at_breakpoint(
            process, init_bkpt)

        self.assertTrue(len(threads) == 1)

        lldbutil.check_expression(self, self.frame(), "in_class_a", "20", use_summary=False)
        lldbutil.check_expression(self, self.frame(), "self.in_class_a", "20", use_summary=False)

        # Disable the init breakpoint so we don't have to hit it again:
        init_bkpt.SetEnabled(False)

        # Now continue to shadowed_in_a:
        self.continue_to_bkpt(process, shadow_a_bkpt)

        lldbutil.check_expression(self, self.frame(), "in_class_a", "10", use_summary=False)
        lldbutil.check_expression(self, self.frame(), "self.in_class_a", "20", use_summary=False)
        lldbutil.check_expression(self, self.frame(), "self.also_in_a", "21", use_summary=False)
        lldbutil.check_expression(self, self.frame(), "also_in_a", "21", use_summary=False)

        shadow_a_bkpt.SetEnabled(False)

        # Now set a breakpoint in the static_method and run to there:
        static_bkpt = target.BreakpointCreateBySourceRegex(
            'In class function', self.main_source_spec)
        self.assertTrue(static_bkpt.GetNumLocations() > 0, VALID_BREAKPOINT)

        self.continue_to_bkpt(process, static_bkpt)

        lldbutil.check_expression(self, self.frame(), "input", "10", use_summary=False)
        # This test fails on Bryce.  The self metatype doesn't generate a valid
        # type.
        lldbutil.check_expression(self, self.frame(), "self.return_ten()", "10", use_summary=False)
        static_bkpt.SetEnabled(False)

        # Now run into the setters & getters:
        set_bkpt = target.BreakpointCreateBySourceRegex(
            'In set.', self.main_source_spec)
        self.assertTrue(set_bkpt.GetNumLocations() > 0, VALID_BREAKPOINT)

        self.continue_to_bkpt(process, set_bkpt)
        lldbutil.check_expression(self, self.frame(), "self.backing_int", "10", use_summary=False)
        set_bkpt.SetEnabled(False)

        get_bkpt = target.BreakpointCreateBySourceRegex(
            'In get.', self.main_source_spec)
        self.assertTrue(get_bkpt.GetNumLocations() > 0, VALID_BREAKPOINT)

        self.continue_to_bkpt(process, get_bkpt)
        lldbutil.check_expression(self, self.frame(), "self.backing_int", "41", use_summary=False)
        get_bkpt.SetEnabled(False)

        deinit_bkpt = target.BreakpointCreateBySourceRegex(
            'In deinit.', self.main_source_spec)
        self.assertTrue(deinit_bkpt.GetNumLocations() > 0, VALID_BREAKPOINT)

        self.continue_to_bkpt(process, deinit_bkpt)
        lldbutil.check_expression(self, self.frame(), "self.backing_int", "41", use_summary=False)
        deinit_bkpt.SetEnabled(False)

        # Now let's try the subscript getter & make sure that that works:

        str_sub_get_bkpt = target.BreakpointCreateBySourceRegex(
            'In string subscript getter', self.main_source_spec)
        self.assertTrue(
            str_sub_get_bkpt.GetNumLocations() > 0,
            VALID_BREAKPOINT)

        self.continue_to_bkpt(process, str_sub_get_bkpt)
        lldbutil.check_expression(self, self.frame(), "self.backing_int", "10", use_summary=False)

        str_sub_get_bkpt.SetEnabled(False)

        # Next run into the closure that captures self and make sure that
        # works:
        closure_bkpt = target.BreakpointCreateBySourceRegex(
            'Break here in closure', self.main_source_spec)
        self.assertTrue(closure_bkpt.GetNumLocations() > 0, VALID_BREAKPOINT)

        self.continue_to_bkpt(process, closure_bkpt)
        lldbutil.check_expression(self, self.frame(), "self.backing_int", "10", use_summary=False)
        lldbutil.check_expression(self, self.frame(), "a_string", '"abcde"', use_summary=True)

        # Now set a breakpoint in the struct method and run to there:
        self.continue_by_pattern('Break here in struct')
        lldbutil.check_expression(self, self.frame(), "a", "\"foo\"", use_summary=True)
        lldbutil.check_expression(self, self.frame(), "self.b", "5", use_summary=False)



import contextlib
import os
from os.path import exists
import lldb
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil


def haswell():
    features = subprocess.check_output(["sysctl", "machdep.cpu"])
    return "AVX2" in features.decode("utf-8")


def apple_silicon():
    features = subprocess.check_output(["sysctl", "machdep.cpu"])
    return "Apple M" in features.decode("utf-8")


def rosetta_debugserver_installed():
    return exists("/Library/Apple/usr/libexec/oah/debugserver")


class TestLaunchProcessPosixSpawn(TestBase):
    NO_DEBUG_INFO_TESTCASE = True

    def no_haswell(self):
        if not haswell():
            return "Current CPU is not Haswell"
        return None

    def no_apple_silicon(self):
        if not apple_silicon():
            return "Current CPU is not Apple Silicon"
        return None

    def run_arch(self, exe, arch):
        self.runCmd("target create -arch {} {}".format(arch, exe))
        self.runCmd("run")

        process = self.dbg.GetSelectedTarget().process
        self.assertState(process.GetState(), lldb.eStateExited)
        self.assertIn("slice: {}".format(arch), process.GetSTDOUT(1000))

    @skipUnlessDarwin
    @skipIfDarwinEmbedded
    @skipIfLLVMTargetMissing("AArch64")
    @skipIfLLVMTargetMissing("X86")
    @skipTestIfFn(no_haswell)
    def test_haswell(self):
        self.build()
        exe = self.getBuildArtifact("fat.out")
        self.run_arch(exe, "x86_64")
        self.run_arch(exe, "x86_64h")

    @skipUnlessDarwin
    @skipIfDarwinEmbedded
    @skipIfLLVMTargetMissing("AArch64")
    @skipIfLLVMTargetMissing("X86")
    @skipTestIfFn(no_apple_silicon)
    def test_apple_silicon(self):
        self.build()
        exe = self.getBuildArtifact("fat.out")
        if rosetta_debugserver_installed():
            self.run_arch(exe, "x86_64")
        self.run_arch(exe, "arm64")



"""Test printing ObjC objects that use unbacked properties - so that the static ivar offsets are incorrect."""


import lldb
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil


class TestObjCIvarStripped(TestBase):
    def setUp(self):
        # Call super's setUp().
        TestBase.setUp(self)
        # Find the line numbers to break inside main().
        self.main_source = "main.m"
        self.stop_line = line_number(self.main_source, "// Set breakpoint here.")

    @skipIf(
        debug_info=no_match("dsym"),
        bugnumber="This test requires a stripped binary and a dSYM",
    )
    @add_test_categories(["pyapi"])
    def test_with_python_api(self):
        """Test that we can find stripped Objective-C ivars in the runtime"""
        self.build()
        exe = self.getBuildArtifact("a.out.stripped")

        target = self.dbg.CreateTarget(exe)
        self.assertTrue(target, VALID_TARGET)

        self.dbg.HandleCommand("add-dsym " + self.getBuildArtifact("a.out.dSYM"))

        breakpoint = target.BreakpointCreateByLocation(self.main_source, self.stop_line)
        self.assertTrue(
            breakpoint.IsValid() and breakpoint.GetNumLocations() > 0, VALID_BREAKPOINT
        )

        process = target.LaunchSimple(None, None, self.get_process_working_directory())
        self.assertTrue(process, "Created a process.")
        self.assertEqual(process.GetState(), lldb.eStateStopped, "Stopped it too.")

        thread_list = lldbutil.get_threads_stopped_at_breakpoint(process, breakpoint)
        self.assertEqual(len(thread_list), 1)
        thread = thread_list[0]

        frame = thread.GetFrameAtIndex(0)
        self.assertTrue(frame, "frame 0 is valid")

        # Test the expression for mc->_foo

        error = lldb.SBError()

        ivar = frame.EvaluateExpression("(mc->_foo)")
        self.assertTrue(ivar, "Got result for mc->_foo")
        ivar_value = ivar.GetValueAsSigned(error)
        self.assertSuccess(error)
        self.assertEqual(ivar_value, 3)



"""
Test that an SBValue can update and format itself as its type changes
"""
import lldb
import lldbsuite.test.decorators as decorators
import lldbsuite.test.lldbtest as lldbtest
import os
import unittest2


class TestSBValueUpdates(lldbtest.TestBase):

    mydir = lldbtest.TestBase.compute_mydir(__file__)

    @decorators.swiftTest
    def test_update_and_format_with_type_change(self):
        """Test that an SBValue can update and format itself as its type
        changes"""
        self.build()
        self.do_test()

    def setUp(self):
        lldbtest.TestBase.setUp(self)
        self.main_source = "main.swift"
        self.main_source_spec = lldb.SBFileSpec(self.main_source)

    def do_test(self):
        """Test that an SBValue can update and format itself as its type
        changes"""
        exe_name = "a.out"
        exe = self.getBuildArtifact(exe_name)

        # Create the target
        target = self.dbg.CreateTarget(exe)
        self.assertTrue(target, lldbtest.VALID_TARGET)

        # Set the breakpoints
        breakpoint = target.BreakpointCreateBySourceRegex(
            'break here', self.main_source_spec)
        self.assertTrue(
            breakpoint.GetNumLocations() > 0, lldbtest.VALID_BREAKPOINT)

        # Launch the process, and do not stop at the entry point.
        self.runCmd("run")

        var_x = self.frame().FindVariable("x", lldb.eDynamicCanRunTarget)
        var_x.SetPreferSyntheticValue(True)

        self.assertTrue(var_x.GetValue() == "1")

        self.runCmd("continue")

        var_x = self.frame().FindVariable("x", lldb.eDynamicCanRunTarget)
        var_x.SetPreferSyntheticValue(True)

        self.assertTrue(var_x.GetSummary() == '"hi"')

        self.runCmd("continue")

        var_x = self.frame().FindVariable("x", lldb.eDynamicCanRunTarget)
        var_x.SetPreferSyntheticValue(True)

        self.assertTrue(var_x.GetValue() == "5")


if __name__ == '__main__':
    import atexit
    lldb.SBDebugger.Initialize()
    atexit.register(lldb.SBDebugger.Terminate)
    unittest2.main()



"""
Tests calling builtin functions using expression evaluation.
"""


import lldb
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil


class ExprCommandCallBuiltinFunction(TestBase):
    # Builtins are expanded by Clang, so debug info shouldn't matter.
    NO_DEBUG_INFO_TESTCASE = True

    def test(self):
        self.build()

        target = self.createTestTarget()

        self.expect_expr("__builtin_isinf(0.0f)", result_type="int", result_value="0")
        self.expect_expr(
            "__builtin_isnormal(0.0f)", result_type="int", result_value="0"
        )
        self.expect_expr("__builtin_constant_p(1)", result_type="int", result_value="1")
        self.expect_expr("__builtin_abs(-14)", result_type="int", result_value="14")



from lldbsuite.test.gdbclientutils import *
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbgdbclient import GDBPlatformClientTestBase


class TestGDBRemotePlatformFile(GDBPlatformClientTestBase):
    def test_file(self):
        """Test mock operations on a remote file"""

        class Responder(MockGDBServerResponder):
            def vFile(self, packet):
                if packet.startswith("vFile:open:"):
                    return "F10"
                elif packet.startswith("vFile:pread:"):
                    return "Fd;frobnicator"
                elif packet.startswith("vFile:pwrite:"):
                    return "Fa"
                elif packet.startswith("vFile:close:"):
                    return "F0"
                return "F-1,58"

        self.server.responder = Responder()

        self.match(
            "platform file open /some/file.txt -v 0755", [r"File Descriptor = 16"]
        )
        self.match(
            "platform file read 16 -o 11 -c 13",
            [r"Return = 11\nData = \"frobnicator\""],
        )
        self.match("platform file write 16 -o 11 -d teststring", [r"Return = 10"])
        self.match("platform file close 16", [r"file 16 closed."])
        self.assertPacketLogContains(
            [
                "vFile:open:2f736f6d652f66696c652e747874,00000202,000001ed",
                "vFile:pread:10,d,b",
                "vFile:pwrite:10,b,teststring",
                "vFile:close:10",
            ]
        )

    def test_file_fail(self):
        """Test mocked failures of remote operations"""

        class Responder(MockGDBServerResponder):
            def vFile(self, packet):
                # use ENOSYS as this constant differs between GDB Remote
                # Protocol and Linux, so we can test the translation
                return "F-1,58"

        self.server.responder = Responder()

        self.match(
            "platform file open /some/file.txt -v 0755",
            [r"error: Function not implemented"],
            error=True,
        )
        self.match(
            "platform file read 16 -o 11 -c 13",
            [r"error: Function not implemented"],
            error=True,
        )
        self.match(
            "platform file write 16 -o 11 -d teststring",
            [r"error: Function not implemented"],
            error=True,
        )
        self.match(
            "platform file close 16", [r"error: Function not implemented"], error=True
        )
        self.assertPacketLogContains(
            [
                "vFile:open:2f736f6d652f66696c652e747874,00000202,000001ed",
                "vFile:pread:10,d,b",
                "vFile:pwrite:10,b,teststring",
                "vFile:close:10",
            ]
        )

    def test_file_size(self):
        """Test 'platform get-size'"""

        class Responder(MockGDBServerResponder):
            def vFile(self, packet):
                return "F1000"

        self.server.responder = Responder()

        self.match(
            "platform get-size /some/file.txt",
            [r"File size of /some/file\.txt \(remote\): 4096"],
        )
        self.assertPacketLogContains(
            [
                "vFile:size:2f736f6d652f66696c652e747874",
            ]
        )

    def test_file_size_fallback(self):
        """Test 'platform get-size fallback to vFile:fstat'"""

        class Responder(MockGDBServerResponder):
            def vFile(self, packet):
                if packet.startswith("vFile:open:"):
                    return "F5"
                elif packet.startswith("vFile:fstat:"):
                    return "F40;" + 28 * "\0" + "\0\0\0\0\0\1\2\3" + 28 * "\0"
                if packet.startswith("vFile:close:"):
                    return "F0"
                return ""

        self.server.responder = Responder()

        self.match(
            "platform get-size /some/file.txt",
            [r"File size of /some/file\.txt \(remote\): 66051"],
        )
        self.assertPacketLogContains(
            [
                "vFile:size:2f736f6d652f66696c652e747874",
                "vFile:open:2f736f6d652f66696c652e747874,00000000,00000000",
                "vFile:fstat:5",
                "vFile:close:5",
            ]
        )

        self.runCmd("platform disconnect")

        # For a new onnection, we should attempt vFile:size once again.
        server2 = MockGDBServer(self.server_socket_class())
        server2.responder = Responder()
        server2.start()
        self.addTearDownHook(lambda: server2.stop())
        self.runCmd("platform connect " + server2.get_connect_url())
        self.match(
            "platform get-size /other/file.txt",
            [r"File size of /other/file\.txt \(remote\): 66051"],
        )

        self.assertPacketLogContains(
            [
                "vFile:size:2f6f746865722f66696c652e747874",
                "vFile:open:2f6f746865722f66696c652e747874,00000000,00000000",
                "vFile:fstat:5",
                "vFile:close:5",
            ],
            log=server2.responder.packetLog,
        )

    @skipIfWindows
    def test_file_permissions(self):
        """Test 'platform get-permissions'"""

        class Responder(MockGDBServerResponder):
            def vFile(self, packet):
                return "F1a4"

        self.server.responder = Responder()

        self.match(
            "platform get-permissions /some/file.txt",
            [r"File permissions of /some/file\.txt \(remote\): 0o0644"],
        )
        self.assertPacketLogContains(
            [
                "vFile:mode:2f736f6d652f66696c652e747874",
            ]
        )

    @skipIfWindows
    def test_file_permissions_fallback(self):
        """Test 'platform get-permissions' fallback to fstat"""

        class Responder(MockGDBServerResponder):
            def vFile(self, packet):
                if packet.startswith("vFile:open:"):
                    return "F5"
                elif packet.startswith("vFile:fstat:"):
                    return "F40;" + 8 * "\0" + "\0\0\1\xA4" + 52 * "\0"
                if packet.startswith("vFile:close:"):
                    return "F0"
                return ""

        self.server.responder = Responder()

        try:
            self.match(
                "platform get-permissions /some/file.txt",
                [r"File permissions of /some/file\.txt \(remote\): 0o0644"],
            )
            self.assertPacketLogContains(
                [
                    "vFile:mode:2f736f6d652f66696c652e747874",
                    "vFile:open:2f736f6d652f66696c652e747874,00000000,00000000",
                    "vFile:fstat:5",
                    "vFile:close:5",
                ]
            )
        finally:
            self.dbg.GetSelectedPlatform().DisconnectRemote()

    def test_file_exists(self):
        """Test 'platform file-exists'"""

        class Responder(MockGDBServerResponder):
            def vFile(self, packet):
                return "F,1"

        self.server.responder = Responder()

        self.match(
            "platform file-exists /some/file.txt",
            [r"File /some/file\.txt \(remote\) exists"],
        )
        self.assertPacketLogContains(
            [
                "vFile:exists:2f736f6d652f66696c652e747874",
            ]
        )

    def test_file_exists_not(self):
        """Test 'platform file-exists' with non-existing file"""

        class Responder(MockGDBServerResponder):
            def vFile(self, packet):
                return "F,0"

        self.server.responder = Responder()

        self.match(
            "platform file-exists /some/file.txt",
            [r"File /some/file\.txt \(remote\) does not exist"],
        )
        self.assertPacketLogContains(
            [
                "vFile:exists:2f736f6d652f66696c652e747874",
            ]
        )

    def test_file_exists_fallback(self):
        """Test 'platform file-exists' fallback to open"""

        class Responder(MockGDBServerResponder):
            def vFile(self, packet):
                if packet.startswith("vFile:open:"):
                    return "F5"
                if packet.startswith("vFile:close:"):
                    return "F0"
                return ""

        self.server.responder = Responder()

        self.match(
            "platform file-exists /some/file.txt",
            [r"File /some/file\.txt \(remote\) exists"],
        )
        self.assertPacketLogContains(
            [
                "vFile:exists:2f736f6d652f66696c652e747874",
                "vFile:open:2f736f6d652f66696c652e747874,00000000,00000000",
                "vFile:close:5",
            ]
        )

    def test_file_exists_not_fallback(self):
        """Test 'platform file-exists' fallback to open with non-existing file"""

        class Responder(MockGDBServerResponder):
            def vFile(self, packet):
                if packet.startswith("vFile:open:"):
                    return "F-1,2"
                return ""

        self.server.responder = Responder()

        self.match(
            "platform file-exists /some/file.txt",
            [r"File /some/file\.txt \(remote\) does not exist"],
        )
        self.assertPacketLogContains(
            [
                "vFile:exists:2f736f6d652f66696c652e747874",
                "vFile:open:2f736f6d652f66696c652e747874,00000000,00000000",
            ]
        )



# TestSwiftObjCOptionals.py
#
# This source file is part of the Swift.org open source project
#
# Copyright (c) 2014 - 2016 Apple Inc. and the Swift project authors
# Licensed under Apache License v2.0 with Runtime Library Exception
#
# See https://swift.org/LICENSE.txt for license information
# See https://swift.org/CONTRIBUTORS.txt for the list of Swift project authors
#
# ------------------------------------------------------------------------------
"""
Check formatting for T? and T! when T is an ObjC type
"""
import lldb
from lldbsuite.test.lldbtest import *
from lldbsuite.test.decorators import *
import lldbsuite.test.lldbutil as lldbutil
import os
import unittest2


class TestSwiftObjCOptionalType(TestBase):
    @swiftTest
    @skipUnlessDarwin
    def test_swift_objc_optional_type(self):
        """Check formatting for T? and T! when T is an ObjC type"""
        self.build()
        self.do_check_consistency()
        self.do_check_visuals()
        self.do_check_api()

    def do_check_consistency(self):
        """Check formatting for T? and T! when T is an ObjC type"""
        lldbutil.run_to_source_breakpoint(
            self, 'Set breakpoint here', lldb.SBFileSpec('main.swift'))

    def do_check_visuals(self):
        """Check formatting for T? and T! when T is an ObjC type"""
        self.expect(
            "frame variable optColor_Some",
            substrs=['Color?) optColor_Some = 0x'])
        self.expect(
            "frame variable uoptColor_Some",
            substrs=['Color?) uoptColor_Some = 0x'])

        self.expect("frame variable optColor_None", substrs=['nil'])
        self.expect("frame variable uoptColor_None", substrs=['nil'])

    def do_check_api(self):
        """Check formatting for T? and T! when T is an ObjC type"""
        optColor_Some = self.frame().FindVariable("optColor_Some")

        # SwiftOptionalSyntheticFrontEnd passes GetNumChildren
        # through to the .some object.  NSColor has no children.
        lldbutil.check_variable(
            self,
            optColor_Some,
            use_dynamic=False,
            num_children=0)
        uoptColor_Some = self.frame().FindVariable("uoptColor_Some")
        lldbutil.check_variable(
            self,
            uoptColor_Some,
            use_dynamic=False,
            num_children=0)




"""
Test ThreadSanitizer when multiple different issues are found.
"""

import lldb
from lldbsuite.test.lldbtest import *
from lldbsuite.test.decorators import *
import lldbsuite.test.lldbutil as lldbutil
import json


class TsanMultipleTestCase(TestBase):
    @expectedFailureAll(
        oslist=["linux"],
        bugnumber="non-core functionality, need to reenable and fix later (DES 2014.11.07)",
    )
    @expectedFailureNetBSD
    @skipIfFreeBSD  # llvm.org/pr21136 runtimes not yet available by default
    @skipIfRemote
    @skipUnlessThreadSanitizer
    @add_test_categories(["objc"])
    def test(self):
        self.build()
        self.tsan_tests()

    def tsan_tests(self):
        exe = self.getBuildArtifact("a.out")
        self.expect("file " + exe, patterns=["Current executable set to .*a.out"])

        self.runCmd("env TSAN_OPTIONS=abort_on_error=0")

        self.runCmd("run")

        stop_reason = (
            self.dbg.GetSelectedTarget().process.GetSelectedThread().GetStopReason()
        )
        if stop_reason == lldb.eStopReasonExec:
            # On OS X 10.10 and older, we need to re-exec to enable
            # interceptors.
            self.runCmd("continue")

        report_count = 0
        while (
            self.dbg.GetSelectedTarget().process.GetSelectedThread().GetStopReason()
            == lldb.eStopReasonInstrumentation
        ):
            report_count += 1

            stop_description = (
                self.dbg.GetSelectedTarget()
                .process.GetSelectedThread()
                .GetStopDescription(100)
            )

            self.assertTrue(
                (stop_description == "Data race detected")
                or (stop_description == "Use of deallocated memory detected")
                or (stop_description == "Thread leak detected")
                or (
                    stop_description
                    == "Use of an uninitialized or destroyed mutex detected"
                )
                or (
                    stop_description
                    == "Unlock of an unlocked mutex (or by a wrong thread) detected"
                )
            )

            self.expect(
                "thread info -s",
                "The extended stop info should contain the TSan provided fields",
                substrs=["instrumentation_class", "description", "mops"],
            )

            output_lines = self.res.GetOutput().split("\n")
            json_line = "\n".join(output_lines[2:])
            data = json.loads(json_line)
            self.assertEqual(data["instrumentation_class"], "ThreadSanitizer")

            backtraces = (
                self.dbg.GetSelectedTarget()
                .process.GetSelectedThread()
                .GetStopReasonExtendedBacktraces(
                    lldb.eInstrumentationRuntimeTypeThreadSanitizer
                )
            )
            self.assertTrue(backtraces.GetSize() >= 1)

            self.runCmd("continue")

        self.assertEqual(
            self.dbg.GetSelectedTarget().process.GetState(),
            lldb.eStateExited,
            PROCESS_EXITED,
        )



# TestSwiftProtocolTypes.py
#
# This source file is part of the Swift.org open source project
#
# Copyright (c) 2014 - 2016 Apple Inc. and the Swift project authors
# Licensed under Apache License v2.0 with Runtime Library Exception
#
# See https://swift.org/LICENSE.txt for license information
# See https://swift.org/CONTRIBUTORS.txt for the list of Swift project authors
#
# ------------------------------------------------------------------------------
"""
Test support for protocol types
"""
import lldb
from lldbsuite.test.lldbtest import *
from lldbsuite.test.decorators import *
import lldbsuite.test.lldbutil as lldbutil
import unittest2


class TestSwiftProtocolTypes(TestBase):
    @swiftTest
    def test_swift_protocol_types(self):
        """Test support for protocol types"""
        self.build()
        lldbutil.run_to_source_breakpoint(self, 'Set breakpoint here',
                                          lldb.SBFileSpec('main.swift'))

        self.expect("frame variable --dynamic-type no-dynamic-values"
                    " --raw-output --show-types loc2d",
                    substrs=['PointUtils) loc2d =',
                             '(Builtin.RawPointer) payload_data_0 = 0x',
                             '(Builtin.RawPointer) payload_data_1 = 0x',
                             '(Builtin.RawPointer) payload_data_2 = 0x',
                             '(Any.Type) metadata = 0x',
                             '(Builtin.RawPointer) wtable = 0x'])
 
        self.expect("frame variable loc2d",
                    substrs=['Point2D) loc2d =',
                             'x = 1.25', 'y = 2.5'])
 
        self.expect("frame variable --dynamic-type no-dynamic-values"
                    " --raw-output --show-types loc3d",
                    substrs=['PointUtils) loc3d =',
                             '(Builtin.RawPointer) payload_data_0 = 0x',
                             '(Builtin.RawPointer) payload_data_1 = 0x',
                             '(Builtin.RawPointer) payload_data_2 = 0x',
                             '(Any.Type) metadata = 0x',
                             '(Builtin.RawPointer) wtable = 0x'])
 
        self.expect(
            "frame variable loc3d",
            substrs=[
                'Point3D) loc3d = 0x',
                'x = 1.25',
                'y = 2.5',
                'z = 1.25'])
 
        self.expect("expression --dynamic-type no-dynamic-values"
                    " --raw-output --show-types -- loc2d",
                    substrs=['PointUtils) $R',
                             '(Builtin.RawPointer) payload_data_0 = 0x',
                             '(Builtin.RawPointer) payload_data_1 = 0x',
                             '(Builtin.RawPointer) payload_data_2 = 0x',
                             '(Any.Type) metadata = 0x',
                             '(Builtin.RawPointer) wtable = 0x'])
 
        self.expect("expression -- loc2d",
                    substrs=['Point2D) $R',
                             'x = 1.25', 'y = 2.5'])
 
        self.expect("expression --dynamic-type no-dynamic-values"
                    " --raw-output --show-types -- loc3dCB",
                    substrs=['PointUtils & Swift.AnyObject) $R',
                             '(Builtin.RawPointer) object = 0x',
                             '(Builtin.RawPointer) wtable = 0x'])
 
        self.expect("expression -- loc3dCB",
                    substrs=['Point3D) $R', 'x = 1.25', 'y = 2.5', 'z = 1.25'])

        self.expect("expression --dynamic-type no-dynamic-values"
                    " --raw-output --show-types -- loc3dSuper",
                    substrs=['(a.PointSuperclass & a.PointUtils) $R',
                             '(a.PointSuperclass) object = 0x',
                             '(Swift.Int) superData = ',
                             '(Builtin.RawPointer) wtable = 0x'])

        self.expect("expression -- loc3dSuper",
                    substrs=['Point3D) $R', 'x = 1.25', 'y = 2.5', 'z = 1.25'])




# TestSwiftRewriteClangPaths.py
#
# This source file is part of the Swift.org open source project
#
# Copyright (c) 2018 Apple Inc. and the Swift project authors
# Licensed under Apache License v2.0 with Runtime Library Exception
#
# See https://swift.org/LICENSE.txt for license information
# See https://swift.org/CONTRIBUTORS.txt for the list of Swift project authors
#
# ------------------------------------------------------------------------------

import lldb
from lldbsuite.test.lldbtest import *
from lldbsuite.test.decorators import *
import lldbsuite.test.lldbutil as lldbutil
import os
import unittest2
import shutil

class TestSwiftRewriteClangPaths(TestBase):
    # Don't run ClangImporter tests if Clangimporter is disabled.
    @skipIf(setting=('symbols.use-swift-clangimporter', 'false'))
    @skipIf(setting=('symbols.swift-precise-compiler-invocation', 'false'))
    @skipUnlessDarwin
    @swiftTest
    @skipIf(debug_info=no_match(["dsym"]))
    def testWithRemap(self):
        self.dotest(True)

    # Don't run ClangImporter tests if Clangimporter is disabled.
    @skipIf(setting=('symbols.use-swift-clangimporter', 'false'))
    @skipIf(setting=('symbols.swift-precise-compiler-invocation', 'false'))
    @skipUnlessDarwin
    @swiftTest
    @skipIf(debug_info=no_match(["dsym"]))
    def testWithoutRemap(self):
        self.dotest(False)

    def find_plist(self):
        import glob
        plist = self.getBuildArtifact("libFoo.dylib.dSYM/Contents/Resources/*.plist")
        lst = glob.glob(plist)
        self.assertTrue(len(lst) == 1)
        return lst[0]
        
    def dotest(self, remap):
        self.build()
        log = self.getBuildArtifact("types.log")
        self.runCmd('log enable lldb types -f "%s"' % log)

        # To ensure the module is rebuilt remove the cache to avoid caching.
        mod_cache = self.getBuildArtifact("my-clang-modules-cache")
        if os.path.isdir(mod_cache):
          shutil.rmtree(mod_cache)
        self.runCmd('settings set symbols.clang-modules-cache-path "%s"'
                    % mod_cache)
        self.runCmd("settings set symbols.use-swift-dwarfimporter false")

        botdir = os.path.realpath(self.getBuildArtifact("buildbot"))
        userdir = os.path.realpath(self.getBuildArtifact("user"))
        self.assertFalse(os.path.isdir(botdir))
        self.assertTrue(os.path.isdir(userdir))
        plist = self.find_plist()
        self.assertTrue(os.path.isfile(plist))
        if remap:
            self.runCmd("settings set target.source-map %s %s %s %s" %
                        (botdir, userdir, '/nonexisting-rootdir', userdir))
        else:
            # Also delete the remapping plist from the .dSYM to verify
            # that this doesn't work by happy accident without it.
            os.remove(plist)

        # Create the target
        target, process, thread, bkpt = lldbutil.run_to_source_breakpoint(
            self, 'break here', lldb.SBFileSpec('Foo.swift'),
            extra_images=['Foo'])

        if remap:
            comment = "returns correct value"
            self.expect("expression foo", comment, substrs=["x", "23"])
            self.expect("expression bar", comment, substrs=["y", "42"])
            self.expect("fr var foo", comment, substrs=["x", "23"])
            self.expect("fr var bar", comment, substrs=["y", "42"])
            self.assertTrue(os.path.isdir(mod_cache), "module cache exists")
        else:
            self.expect("expression foo", error=True)

        # Scan through the types log.
        suffix = "REMAP" if remap else "NORMAL"
        self.filecheck('platform shell cat "%s"' % log, __file__,
                       '--check-prefix=CHECK_' + suffix)
# CHECK_REMAP-NOT: remapped -iquote
# CHECK_REMAP-NOT: error:{{.*}}Foo
# CHECK_NORMAL: error:{{.*}}Foo
# CHECK_REMAP-DAG: SwiftASTContextForExpressions(module: "Foo"{{.*}}/buildbot/Foo{{.*}} -> {{.*}}/user/Foo
# CHECK_REMAP-DAG: SwiftASTContextForExpressions(module: "Foo"{{.*}}/buildbot/iquote-path{{.*}} -> {{.*}}/user/iquote-path
# CHECK_REMAP-DAG: SwiftASTContextForExpressions(module: "Foo"{{.*}}/buildbot/I-double{{.*}} -> {{.*}}/user/I-double
# CHECK_REMAP-DAG: SwiftASTContextForExpressions(module: "Foo"{{.*}}/buildbot/I-single{{.*}} -> {{.*}}/user/I-single
# CHECK_REMAP-DAG: SwiftASTContextForExpressions(module: "Foo"{{.*}}/buildbot/Frameworks{{.*}} -> {{.*}}/user/Frameworks
# CHECK_REMAP-DAG: SwiftASTContextForExpressions(module: "Foo"{{.*}}/nonexisting-rootdir{{.*}} -> {{.*}}/user
# CHECK_REMAP-DAG: SwiftASTContextForExpressions(module: "Foo"{{.*}}/buildbot/Foo/overlay.yaml{{.*}} -> {{.*}}/user/Foo/overlay.yaml



"""
Test setting a breakpoint by line and column.
"""


import lldb
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil


class BreakpointByLineAndColumnTestCase(TestBase):
    def testBreakpointSpecWithLine(self):
        self.build()
        target = self.createTestTarget()

        # This one should work:
        lldbutil.run_break_set_by_file_colon_line(
            self, "main.c:11", "main.c", 11, num_expected_locations=1
        )
        # Let's try an illegal specifier to make sure the command fails.  I'm not being exhaustive
        # since the UnitTest has more bad patterns.  I'm just testing that if the SetFromString
        # fails, we propagate the error.
        self.expect("break set -y 'foo.c'", error=True)

    ## Skip gcc version less 7.1 since it doesn't support -gcolumn-info
    @skipIf(compiler="gcc", compiler_version=["<", "7.1"])
    def testBreakpointByLine(self):
        self.build()
        target = self.createTestTarget()

        main_c = lldb.SBFileSpec("main.c")
        lldbutil.run_break_set_by_file_colon_line(
            self, "main.c:11:50", "main.c", 11, num_expected_locations=1
        )



import lldb
from lldbsuite.test.lldbtest import *
from lldbsuite.test.decorators import *
import lldbsuite.test.lldbutil as lldbutil
import os
import unittest2


class TestSwiftMacCatalyst(TestBase):
    @swiftTest
    @skipIf(macos_version=["<", "10.15"])
    @skipUnlessDarwin
    @skipIfDarwinEmbedded
    def test_macCatalyst(self):
        """Test the ${arch}-apple-ios-macabi target.
           Note that this test only works when build-script
           is being invoked with --maccatalyst!
        """
        self.build()

        types_log = self.getBuildArtifact('types.log')
        self.runCmd('log enable lldb types -f "%s"' % types_log)

        lldbutil.run_to_source_breakpoint(self, "break here",
                                          lldb.SBFileSpec('main.swift'))
        self.expect("image list -t -b",
                    patterns=["-apple-ios14.0.0-macabi a\.out",
                              "-apple-ios.*-macabi Foundation",
                              "-apple-.* libswiftCore",
                              "-apple-macosx.* libcompiler_rt.dylib"])
        self.expect("fr v s", substrs=["Hello macCatalyst"])
        expr_log = self.getBuildArtifact("expr.log")
        self.expect('log enable lldb expr -f "%s"' % expr_log)
        self.expect("expression s", substrs=["Hello macCatalyst"])
        import io
        expr_logfile = io.open(expr_log, "r", encoding='utf-8')

        import re
        availability_re = re.compile(r'@available\(macCatalyst 1.*, \*\)')
        found = False
        for line in expr_logfile:
            self.trace(line)
            if availability_re.search(line):
               found = True
               break
        self.assertTrue(found)

        found_prebuilt = False
        found_sdk = False
        import io
        types_logfile = io.open(types_log, "r", encoding='utf-8')
        for line in types_logfile:
            if 'Using prebuilt Swift module cache path: ' in line:
                self.assertTrue(line.endswith('/macosx/prebuilt-modules\n'),
                                'unexpected prebuilt cache path: ' + line)
                found_prebuilt = True
            if 'SDK path ' in line:
                self.assertTrue('SDKs/MacOSX' in line,
                                'picked non-macOS SDK:' + line)
                found_sdk = True
        self.assertTrue(found_prebuilt, 'prebuilt cache path log entry not found')
        self.assertTrue(found_sdk, 'SDK path log entry not found')




"""
Test that Swift types are displayed correctly in C++
"""
from lldbsuite.test.lldbtest import *
from lldbsuite.test.decorators import *


class TestSwiftBackwardInteropStepping(TestBase):

    def setup(self, bkpt_str):
        self.build()
        
        _, _, thread, _ = lldbutil.run_to_source_breakpoint(
            self, bkpt_str, lldb.SBFileSpec('main.cpp'))
        return thread


    def check_step_in(self, thread, caller, callee):
        name = thread.frames[0].GetFunctionName()
        self.assertIn(caller, name)
        thread.StepInto()
        name = thread.frames[0].GetFunctionName()
        self.assertIn(callee, name)
        thread.StepOut()
        name = thread.frames[0].GetFunctionName()
        self.assertIn(caller, name)

    def check_step_over(self, thread, func):
        name = thread.frames[0].GetFunctionName()
        self.assertIn(func, name)
        thread.StepOver()
        name = thread.frames[0].GetFunctionName()
        self.assertIn(func, name)

    @swiftTest
    def test_func_step_in(self):
        thread = self.setup('Break here for func')
        self.check_step_in(thread, 'testFunc', 'swiftFunc')
        
    @swiftTest
    def test_func_step_over(self):
        thread = self.setup('Break here for func')
        self.check_step_over(thread, 'testFunc')

    @swiftTest
    def test_method_step_in_class(self):
        thread = self.setup('Break here for method - class')
        self.check_step_in(thread, 'testMethod', 'SwiftClass.swiftMethod')
        
    @swiftTest
    def test_method_step_over_class(self):
        thread = self.setup('Break here for method - class')
        self.check_step_over(thread, 'testMethod')

    @expectedFailureAll(bugnumber="rdar://106670255")
    @swiftTest
    def test_init_step_in_class(self):
        thread = self.setup('Break here for constructor - class')
        self.check_step_in(thread, 'testConstructor', 'SwiftClass.init')
        
    @swiftTest
    def test_init_step_over_class(self):
        thread = self.setup('Break here for constructor - class')
        self.check_step_over(thread, 'testConstructor')

    @swiftTest
    def test_static_method_step_in_class(self):
        thread = self.setup('Break here for static method - class')
        self.check_step_in(thread, 'testStaticMethod', 'SwiftClass.swiftStaticMethod')
        
    @swiftTest
    def test_static_method_step_over_class(self):
        thread = self.setup('Break here for static method - class')
        self.check_step_over(thread, 'testStaticMethod')

    @swiftTest
    def test_getter_step_in_class(self):
        thread = self.setup('Break here for getter - class')
        self.check_step_in(thread, 'testGetter', 'SwiftClass.swiftProperty.getter')
        
    @swiftTest
    def test_getter_step_over_class(self):
        thread = self.setup('Break here for getter - class')
        self.check_step_over(thread, 'testGetter')

    @swiftTest
    def test_setter_step_in_class(self):
        thread = self.setup('Break here for setter - class')
        self.check_step_in(thread, 'testSetter', 'SwiftClass.swiftProperty.setter')
        
    @swiftTest
    def test_setter_step_over_class(self):
        thread = self.setup('Break here for setter - class')
        self.check_step_over(thread, 'testSetter')


    @swiftTest
    def test_overriden_step_in_class(self):
        thread = self.setup('Break here for overridden - class')
        self.check_step_in(thread, 'testOverridenMethod', 'SwiftSubclass.overrideableMethod')
        
    @swiftTest
    def test_overriden_step_over_class(self):
        thread = self.setup('Break here for overridden')
        self.check_step_over(thread, 'testOverridenMethod')

    @swiftTest
    def test_method_step_in_struct_class(self):
        thread = self.setup('Break here for method - struct')
        self.check_step_in(thread, 'testMethod', 'SwiftStruct.swiftMethod')
        
    @swiftTest
    def test_method_step_over_struct_class(self):
        thread = self.setup('Break here for method - struct')
        self.check_step_over(thread, 'testMethod')

    @expectedFailureAll(bugnumber="rdar://106670255")
    @swiftTest
    def test_init_step_in_struct_class(self):
        thread = self.setup('Break here for constructor - struct')
        self.check_step_in(thread, 'testConstructor', 'SwiftStruct.init')
        
    @swiftTest
    def test_init_step_over_struct_class(self):
        thread = self.setup('Break here for constructor - struct')
        self.check_step_over(thread, 'testConstructor')

    @swiftTest
    def test_static_method_step_in_struct(self):
        thread = self.setup('Break here for static method - struct')
        self.check_step_in(thread, 'testStaticMethod', 'SwiftStruct.swiftStaticMethod')
        
    @swiftTest
    def test_static_method_step_over_struct(self):
        thread = self.setup('Break here for static method - struct')
        self.check_step_over(thread, 'testStaticMethod')

    @swiftTest
    def test_getter_step_in_struct(self):
        thread = self.setup('Break here for getter - struct')
        self.check_step_in(thread, 'testGetter', 'SwiftStruct.swiftProperty.getter')
        
    @swiftTest
    def test_getter_step_over_struct(self):
        thread = self.setup('Break here for getter - struct')
        self.check_step_over(thread, 'testGetter')

    @swiftTest
    def test_setter_step_in_struct(self):
        thread = self.setup('Break here for setter - struct')
        self.check_step_in(thread, 'testSetter', 'SwiftStruct.swiftProperty.setter')
        
    @swiftTest
    def test_setter_step_over_struct(self):
        thread = self.setup('Break here for setter - struct')
        self.check_step_over(thread, 'testSetter')




# TestSwiftClassTypes.py
#
# This source file is part of the Swift.org open source project
#
# Copyright (c) 2014 - 2016 Apple Inc. and the Swift project authors
# Licensed under Apache License v2.0 with Runtime Library Exception
#
# See https://swift.org/LICENSE.txt for license information
# See https://swift.org/CONTRIBUTORS.txt for the list of Swift project authors
#
# ------------------------------------------------------------------------------
"""
Test swift Class types
"""
import lldb
from lldbsuite.test.lldbtest import *
from lldbsuite.test.decorators import *
import lldbsuite.test.lldbutil as lldbutil
import os
import unittest2


class TestSwiftClassTypes(TestBase):
    @swiftTest
    def test_swift_class_types(self):
        """Test swift Class types"""
        self.build()
        lldbutil.run_to_source_breakpoint(self, 'Set breakpoint here',
                                          lldb.SBFileSpec('main.swift'))
        self.expect("frame variable --show-types f",
                    substrs=['Foo) f = 0x',
                             'Base) ', '.Base = {',
                             '(String) b = ',
                             '(Int) x = 12',
                             '(Float) y = 2.25'])




# TestSwiftMeta.py
#
# This source file is part of the Swift.org open source project
#
# Copyright (c) 2018 Apple Inc. and the Swift project authors
# Licensed under Apache License v2.0 with Runtime Library Exception
#
# See https://swift.org/LICENSE.txt for license information
# See https://swift.org/CONTRIBUTORS.txt for the list of Swift project authors
#
# ------------------------------------------------------------------------------
"""
Test the Swift test decorator itself.
"""
from lldbsuite.test.decorators import *
import lldbsuite.test.lldbtest as lldbtest
import os

class TestSwiftMeta(lldbtest.TestBase):

    mydir = lldbtest.TestBase.compute_mydir(__file__)

    @swiftTest
    def test_swiftDecorator(self):
        self.assertTrue(self.getDebugInfo() != "gmodules")

    @swiftTest
    def test_swiftBuild(self):
        self.build()
        exe = self.getBuildArtifact()
        dsym = exe+'.dSYM'
        self.assertTrue(os.path.isfile(exe))
        if self.getDebugInfo() == "dwarf":
            self.assertFalse(os.path.isdir(dsym),
                             'testing DWARF, but .dSYM present')
        if self.getDebugInfo() == "dsym":
            self.assertTrue(os.path.isdir(dsym),
                            '.dSYM is missing in dsym config')



# TestSwiftNoProcess.py
#
# This source file is part of the Swift.org open source project
#
# Copyright (c) 2014 - 2016 Apple Inc. and the Swift project authors
# Licensed under Apache License v2.0 with Runtime Library Exception
#
# See https://swift.org/LICENSE.txt for license information
# See https://swift.org/CONTRIBUTORS.txt for the list of Swift project authors
#
# ------------------------------------------------------------------------------
"""
Tests simple swift expressions
"""
import lldb
from lldbsuite.test.lldbtest import *
from lldbsuite.test.decorators import *
import lldbsuite.test.lldbutil as lldbutil
import os
import unittest2


class TestSwiftNoProcess(TestBase):
    def setUp(self):
        TestBase.setUp(self)
        self.main_source = "main.swift"
        self.main_source_spec = lldb.SBFileSpec(self.main_source)

    @swiftTest
    @skipIf(oslist=['linux', 'windows'])
    def test_swift_no_target(self):
        """Tests that we give a reasonable error if we try to run expressions with no target"""
        result = lldb.SBCommandReturnObject()
        ret_val = self.dbg.GetCommandInterpreter().HandleCommand(
            "expression -l swift -- 1 + 2", result)
        self.assertTrue(
            ret_val == lldb.eReturnStatusFailed,
            "Swift expression with no target should fail.")

    @swiftTest
    @skipIf(oslist=['windows'])
    def test_swift_no_process(self):
        """Tests that we give a reasonable error if we try to run expressions with no process"""
        self.build()

        exe_name = "a.out"
        exe = self.getBuildArtifact(exe_name)

        # Create the target
        target = self.dbg.CreateTarget(exe)
        self.assertTrue(target, VALID_TARGET)
        result = lldb.SBCommandReturnObject()
        options = lldb.SBExpressionOptions()
        options.SetLanguage(lldb.eLanguageTypeSwift)
        ret_val = target.EvaluateExpression("1 + 2", options)
        self.assertTrue(ret_val.GetError().Fail(),
                        "Swift expressions with no process should fail.")





"""
Test that we are able to broadcast and receive swift progress events from lldb
"""
import lldb

import lldbsuite.test.lldbutil as lldbutil

from lldbsuite.test.lldbtest import *
from lldbsuite.test.decorators import *

class TestSwiftProgressReporting(TestBase):
    def setUp(self):
        TestBase.setUp(self)
        self.broadcaster = self.dbg.GetBroadcaster()
        self.listener = lldbutil.start_listening_from(self.broadcaster,
                                        lldb.SBDebugger.eBroadcastBitProgress)

    @skipIf(setting=('plugin.typesystem.clang.experimental-redecl-completion', 'true'))
    # Don't run ClangImporter tests if Clangimporter is disabled.
    @skipIf(setting=('symbols.use-swift-clangimporter', 'false'))
    @skipUnlessDarwin
    @swiftTest
    def test_swift_progress_report(self):
        """Test that we are able to fetch swift type-system progress events"""
        self.build()

        target, process, thread, _ = lldbutil.run_to_source_breakpoint(self, 'break here',
                                          lldb.SBFileSpec('main.swift'))

        self.assertGreater(thread.GetNumFrames(), 0)
        frame = thread.GetSelectedFrame()
        self.assertTrue(frame, "Invalid frame.")

        # Resolve variable to exercise the type-system
        self.runCmd("expr boo")
        self.runCmd("v s")

        beacons = [
            "Loading Swift module",
            "Compiling bridging header",
            "Importing modules used in expression",
            "Setting up Swift reflection",
            "Importing Swift module dependencies for main.swift",
            "Importing Swift modules",
            "Importing Swift standard library",
        ]

        importing_swift_reports = []
        while len(beacons):
            event = lldbutil.fetch_next_event(self, self.listener, self.broadcaster)
            ret_args = lldb.SBDebugger.GetProgressFromEvent(event)
            if self.TraceOn():
                print(ret_args[0])

            # When importing Swift modules, make sure that we don't get two reports
            # in a row with the title "Importing Swift modules", i.e. there should be
            # a report with that title followed by a report with that title and details
            # attached.
            if ret_args[0] == "Importing Swift modules":
                next_event = lldbutil.fetch_next_event(self, self.listener, self.broadcaster)
                next_ret_args = lldb.SBDebugger.GetProgressFromEvent(next_event)
                self.assertRegexpMatches(next_ret_args[0], r"Importing Swift modules:+")

            for beacon in beacons:
                if beacon in ret_args[0]:
                    beacons.remove(beacon)



"""
Test lldb exception breakpoint command for CPP.
"""


import lldb
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil


class CPPBreakpointTestCase(TestBase):
    def setUp(self):
        # Call super's setUp().
        TestBase.setUp(self)
        self.source = "exceptions.cpp"
        self.catch_line = line_number(
            self.source, "// This is the line you should stop at for catch"
        )

    @expectedFailureAll(
        oslist=["windows"],
        bugnumber="llvm.org/pr24538, clang-cl does not support throw or catch",
    )
    def test(self):
        """Test lldb exception breakpoint command for CPP."""
        self.build()
        exe = self.getBuildArtifact("a.out")

        # Create a target from the debugger.

        target = self.dbg.CreateTarget(exe)
        self.assertTrue(target, VALID_TARGET)

        exception_bkpt = target.BreakpointCreateForException(
            lldb.eLanguageTypeC_plus_plus, True, True
        )
        self.assertTrue(exception_bkpt, "Made an exception breakpoint")

        # Now run, and make sure we hit our breakpoint:
        process = target.LaunchSimple(None, None, self.get_process_working_directory())
        self.assertTrue(process, "Got a valid process")

        stopped_threads = []
        stopped_threads = lldbutil.get_threads_stopped_at_breakpoint(
            process, exception_bkpt
        )
        self.assertEqual(
            len(stopped_threads), 1, "Stopped at our exception breakpoint."
        )
        thread = stopped_threads[0]
        # Make sure our throw function is still above us on the stack:

        frame_functions = lldbutil.get_function_names(thread)
        self.assertEqual(
            frame_functions.count("throws_exception_on_even(int)"),
            1,
            "Our throw function is still on the stack.",
        )

        # Okay we hit our exception throw breakpoint, now make sure we get our catch breakpoint.
        # One potential complication is that we might hit a couple of the exception breakpoints in getting out of the throw.
        # so loop till we don't see the throws function on the stack.  We should stop one more time for our exception breakpoint
        # and that should be the catch...

        while frame_functions.count("throws_exception_on_even(int)") == 1:
            stopped_threads = lldbutil.continue_to_breakpoint(process, exception_bkpt)
            self.assertEqual(len(stopped_threads), 1)

            thread = stopped_threads[0]
            frame_functions = lldbutil.get_function_names(thread)

        self.assertEqual(
            frame_functions.count("throws_exception_on_even(int)"),
            0,
            "At catch our throw function is off the stack",
        )
        self.assertEqual(
            frame_functions.count("intervening_function(int)"),
            0,
            "At catch our intervening function is off the stack",
        )
        self.assertEqual(
            frame_functions.count("catches_exception(int)"),
            1,
            "At catch our catch function is on the stack",
        )



import lldb
from lldbsuite.test.decorators import *
import lldbsuite.test.lldbtest as lldbtest
import lldbsuite.test.lldbutil as lldbutil
import unittest2
import re

class TestSwiftReflectionLoading(lldbtest.TestBase):
    NO_DEBUG_INFO_TESTCASE = True

    mydir = lldbtest.TestBase.compute_mydir(__file__)

    @swiftTest
    def test(self):
        """Test that reflection metadata is imported"""
        self.build()

        log = self.getBuildArtifact("types.log")
        self.runCmd('log enable lldb types -f "%s"' % log)

        target, process, thread, bkpt = lldbutil.run_to_source_breakpoint(
            self, 'Set breakpoint here', lldb.SBFileSpec('main.swift'),
            extra_images=['dynamic_lib'])
        frame = thread.frames[0]
        var_c = frame.FindVariable("c")
        var_c_x = var_c.GetChildMemberWithName("x")
        lldbutil.check_variable(self, var_c_x, value="23")

        # Scan through the types log.
        import io
        logfile = io.open(log, "r", encoding='utf-8')
        found_exe = 0
        found_lib = 0
        for line in logfile:
            if re.search(r'Adding reflection metadata in .*a\.out', line):
                found_exe += 1
            if re.search(r'Adding reflection metadata in .*dynamic_lib', line):
                found_lib += 1
        self.assertEqual(found_exe, 1)
        self.assertEqual(found_lib, 1)



"""
Fuzz tests an object after the default construction to make sure it does not crash lldb.
"""

import lldb


def fuzz_obj(obj):
    obj.GetProcess()
    listener = lldb.SBListener()
    error = lldb.SBError()
    obj.Launch(listener, None, None, None, None, None, None, 0, True, error)
    obj.LaunchSimple(None, None, None)
    obj.AttachToProcessWithID(listener, 123, error)
    obj.AttachToProcessWithName(listener, "lldb", False, error)
    obj.ConnectRemote(listener, "connect://to/here", None, error)
    obj.GetExecutable()
    obj.GetNumModules()
    obj.GetModuleAtIndex(0xFFFFFFFF)
    obj.GetDebugger()
    filespec = lldb.SBFileSpec()
    obj.FindModule(filespec)
    sc_list = obj.FindFunctions("the_func")
    sc_list = obj.FindFunctions("the_func", lldb.eFunctionNameTypeAny)
    obj.FindFirstType("dont_care")
    obj.FindTypes("dont_care")
    obj.FindFirstType(None)
    obj.GetInstructions(lldb.SBAddress(), bytearray())
    obj.GetSourceManager()
    obj.FindGlobalVariables("my_global_var", 1)
    address = obj.ResolveLoadAddress(0xFFFF)
    obj.ResolveSymbolContextForAddress(address, 0)
    obj.BreakpointCreateByLocation("filename", 20)
    obj.BreakpointCreateByLocation(filespec, 20)
    obj.BreakpointCreateByName("func", None)
    obj.BreakpointCreateByRegex("func.", None)
    obj.BreakpointCreateByAddress(0xF0F0)
    obj.GetNumBreakpoints()
    obj.GetBreakpointAtIndex(0)
    obj.BreakpointDelete(0)
    obj.FindBreakpointByID(0)
    obj.EnableAllBreakpoints()
    obj.DisableAllBreakpoints()
    obj.DeleteAllBreakpoints()
    obj.GetNumWatchpoints()
    obj.GetWatchpointAtIndex(0)
    obj.DeleteWatchpoint(0)
    obj.FindWatchpointByID(0)
    obj.EnableAllWatchpoints()
    obj.DisableAllWatchpoints()
    obj.DeleteAllWatchpoints()
    obj.GetAddressByteSize()
    obj.GetByteOrder()
    obj.GetTriple()
    error = lldb.SBError()
    wp_opts = lldb.SBWatchpointOptions()
    wp_opts.SetWatchpointTypeRead(True)
    wp_opts.SetWatchpointTypeWrite(lldb.eWatchpointWriteTypeOnModify)
    obj.WatchpointCreateByAddress(123, 8, wp_opts, error)
    obj.GetBroadcaster()
    obj.GetDescription(lldb.SBStream(), lldb.eDescriptionLevelBrief)
    obj.Clear()
    for module in obj.module_iter():
        s = str(module)
    for bp in obj.breakpoint_iter():
        s = str(bp)
    for wp in obj.watchpoint_iter():
        s = str(wp)



import shutil
import lldb
from lldbsuite.test.decorators import *
import lldbsuite.test.lldbtest as lldbtest
import lldbsuite.test.lldbutil as lldbutil


class TestCase(lldbtest.TestBase):
    @swiftTest
    def test_missing_explicit_modules(self):
        """Test missing explicit Swift modules and fallback to implicit modules."""
        self.build()

        # This test verifies the case where explicit modules are missing.
        # Remove explicit modules from their place in the module cache.
        mod_cache = self.getBuildArtifact("private-module-cache")
        shutil.rmtree(mod_cache)

        lldbutil.run_to_source_breakpoint(
            self, "Set breakpoint here", lldb.SBFileSpec("main.swift")
        )

        log = self.getBuildArtifact("types.log")
        self.runCmd(f"log enable lldb types -f '{log}'")

        self.expect("expression c", substrs=["hello implicit fallback"])

        self.filecheck(f"platform shell cat {log}", __file__)
        # CHECK: Nonexistent explicit module file



"""
Make sure that "target modules lookup -va $pc" works
"""


import lldb
import lldbsuite.test.lldbutil as lldbutil
from lldbsuite.test.lldbtest import *


class TestImageLookupPCInC(TestBase):
    def test_sample_rename_this(self):
        """There can be many tests in a test case - describe this test here."""
        self.build()
        self.main_source_file = lldb.SBFileSpec("main.c")
        self.sample_test()

    def sample_test(self):
        """Make sure the address expression resolves to the right function"""

        (target, process, thread, bkpt) = lldbutil.run_to_source_breakpoint(
            self, "Set a breakpoint here", self.main_source_file
        )

        self.expect("target modules lookup -va $pc", substrs=["doSomething"])
        self.expect("target modules lookup -va $pc+4", substrs=["doSomething"])




import lldb
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil


class TestCase(TestBase):
    def test(self):
        """
        Tests a forward declared template and a normal template in the same
        executable. GCC/Clang emit very limited debug information for forward
        declared templates that might trip up LLDB.
        """
        self.build()
        lldbutil.run_to_source_breakpoint(
            self, "// break here", lldb.SBFileSpec("main.cpp")
        )

        self.expect_expr("a; b", result_type="Temp<float>")



import lldb
from lldbsuite.test.lldbtest import *
from lldbsuite.test.decorators import *
from lldbsuite.test.gdbclientutils import *
from lldbsuite.test.lldbgdbclient import GDBRemoteTestBase


class TestStopPCs(GDBRemoteTestBase):
    @skipIfXmlSupportMissing
    def test(self):
        class MyResponder(MockGDBServerResponder):
            def haltReason(self):
                return "T02thread:1ff0d;threads:1ff0d,2ff0d;thread-pcs:10001bc00,10002bc00;"

            def threadStopInfo(self, threadnum):
                if threadnum == 0x1FF0D:
                    return "T02thread:1ff0d;threads:1ff0d,2ff0d;thread-pcs:10001bc00,10002bc00;"
                if threadnum == 0x2FF0D:
                    return "T00thread:2ff0d;threads:1ff0d,2ff0d;thread-pcs:10001bc00,10002bc00;"

            def qXferRead(self, obj, annex, offset, length):
                if annex == "target.xml":
                    return (
                        """<?xml version="1.0"?>
                        <target version="1.0">
                          <architecture>i386:x86-64</architecture>
                          <feature name="org.gnu.gdb.i386.core">
                            <reg name="rip" bitsize="64" regnum="0" type="code_ptr" group="general"/>
                          </feature>
                        </target>""",
                        False,
                    )
                else:
                    return None, False

        self.server.responder = MyResponder()
        target = self.dbg.CreateTarget("")
        if self.TraceOn():
            self.runCmd("log enable gdb-remote packets")
            self.addTearDownHook(lambda: self.runCmd("log disable gdb-remote packets"))
        process = self.connect(target)

        self.assertEqual(process.GetNumThreads(), 2)
        th0 = process.GetThreadAtIndex(0)
        th1 = process.GetThreadAtIndex(1)
        self.assertEqual(th0.GetThreadID(), 0x1FF0D)
        self.assertEqual(th1.GetThreadID(), 0x2FF0D)
        self.assertEqual(th0.GetFrameAtIndex(0).GetPC(), 0x10001BC00)
        self.assertEqual(th1.GetFrameAtIndex(0).GetPC(), 0x10002BC00)



# RUN: %PYTHON %s | FileCheck %s

import gc
from mlir.ir import *
from mlir.dialects import func


def run(f):
    print("\nTEST:", f.__name__)
    f()
    gc.collect()
    assert Context._get_live_count() == 0
    return f


# CHECK-LABEL: TEST: testCapsuleConversions
@run
def testCapsuleConversions():
    ctx = Context()
    ctx.allow_unregistered_dialects = True
    with Location.unknown(ctx):
        i32 = IntegerType.get_signless(32)
        value = Operation.create("custom.op1", results=[i32]).result
        value_capsule = value._CAPIPtr
        assert '"mlir.ir.Value._CAPIPtr"' in repr(value_capsule)
        value2 = Value._CAPICreate(value_capsule)
        assert value2 == value


# CHECK-LABEL: TEST: testOpResultOwner
@run
def testOpResultOwner():
    ctx = Context()
    ctx.allow_unregistered_dialects = True
    with Location.unknown(ctx):
        i32 = IntegerType.get_signless(32)
        op = Operation.create("custom.op1", results=[i32])
        assert op.result.owner == op


# CHECK-LABEL: TEST: testBlockArgOwner
@run
def testBlockArgOwner():
    ctx = Context()
    ctx.allow_unregistered_dialects = True
    module = Module.parse(
        r"""
    func.func @foo(%arg0: f32) {
      return
    }""",
        ctx,
    )
    func = module.body.operations[0]
    block = func.regions[0].blocks[0]
    assert block.arguments[0].owner == block


# CHECK-LABEL: TEST: testValueIsInstance
@run
def testValueIsInstance():
    ctx = Context()
    ctx.allow_unregistered_dialects = True
    module = Module.parse(
        r"""
    func.func @foo(%arg0: f32) {
      %0 = "some_dialect.some_op"() : () -> f64
      return
    }""",
        ctx,
    )
    func = module.body.operations[0]
    assert BlockArgument.isinstance(func.regions[0].blocks[0].arguments[0])
    assert not OpResult.isinstance(func.regions[0].blocks[0].arguments[0])

    op = func.regions[0].blocks[0].operations[0]
    assert not BlockArgument.isinstance(op.results[0])
    assert OpResult.isinstance(op.results[0])


# CHECK-LABEL: TEST: testValueHash
@run
def testValueHash():
    ctx = Context()
    ctx.allow_unregistered_dialects = True
    module = Module.parse(
        r"""
    func.func @foo(%arg0: f32) -> f32 {
      %0 = "some_dialect.some_op"(%arg0) : (f32) -> f32
      return %0 : f32
    }""",
        ctx,
    )

    [func] = module.body.operations
    block = func.entry_block
    op, ret = block.operations
    assert hash(block.arguments[0]) == hash(op.operands[0])
    assert hash(op.result) == hash(ret.operands[0])


# CHECK-LABEL: TEST: testValueUses
@run
def testValueUses():
    ctx = Context()
    ctx.allow_unregistered_dialects = True
    with Location.unknown(ctx):
        i32 = IntegerType.get_signless(32)
        module = Module.create()
        with InsertionPoint(module.body):
            value = Operation.create("custom.op1", results=[i32]).results[0]
            op1 = Operation.create("custom.op2", operands=[value])
            op2 = Operation.create("custom.op2", operands=[value])

    # CHECK: Use owner: "custom.op2"
    # CHECK: Use operand_number: 0
    # CHECK: Use owner: "custom.op2"
    # CHECK: Use operand_number: 0
    for use in value.uses:
        assert use.owner in [op1, op2]
        print(f"Use owner: {use.owner}")
        print(f"Use operand_number: {use.operand_number}")


# CHECK-LABEL: TEST: testValueReplaceAllUsesWith
@run
def testValueReplaceAllUsesWith():
    ctx = Context()
    ctx.allow_unregistered_dialects = True
    with Location.unknown(ctx):
        i32 = IntegerType.get_signless(32)
        module = Module.create()
        with InsertionPoint(module.body):
            value = Operation.create("custom.op1", results=[i32]).results[0]
            op1 = Operation.create("custom.op2", operands=[value])
            op2 = Operation.create("custom.op2", operands=[value])
            value2 = Operation.create("custom.op3", results=[i32]).results[0]
            value.replace_all_uses_with(value2)

    assert len(list(value.uses)) == 0

    # CHECK: Use owner: "custom.op2"
    # CHECK: Use operand_number: 0
    # CHECK: Use owner: "custom.op2"
    # CHECK: Use operand_number: 0
    for use in value2.uses:
        assert use.owner in [op1, op2]
        print(f"Use owner: {use.owner}")
        print(f"Use operand_number: {use.operand_number}")


# CHECK-LABEL: TEST: testValuePrintAsOperand
@run
def testValuePrintAsOperand():
    ctx = Context()
    ctx.allow_unregistered_dialects = True
    with Location.unknown(ctx):
        i32 = IntegerType.get_signless(32)
        module = Module.create()
        with InsertionPoint(module.body):
            value = Operation.create("custom.op1", results=[i32]).results[0]
            # CHECK: Value(%[[VAL1:.*]] = "custom.op1"() : () -> i32)
            print(value)

            value2 = Operation.create("custom.op2", results=[i32]).results[0]
            # CHECK: Value(%[[VAL2:.*]] = "custom.op2"() : () -> i32)
            print(value2)

            f = func.FuncOp("test", ([i32, i32], []))
            entry_block1 = Block.create_at_start(f.operation.regions[0], [i32, i32])

            with InsertionPoint(entry_block1):
                value3 = Operation.create("custom.op3", results=[i32]).results[0]
                # CHECK: Value(%[[VAL3:.*]] = "custom.op3"() : () -> i32)
                print(value3)
                value4 = Operation.create("custom.op4", results=[i32]).results[0]
                # CHECK: Value(%[[VAL4:.*]] = "custom.op4"() : () -> i32)
                print(value4)

                f = func.FuncOp("test", ([i32, i32], []))
                entry_block2 = Block.create_at_start(f.operation.regions[0], [i32, i32])
                with InsertionPoint(entry_block2):
                    value5 = Operation.create("custom.op5", results=[i32]).results[0]
                    # CHECK: Value(%[[VAL5:.*]] = "custom.op5"() : () -> i32)
                    print(value5)
                    value6 = Operation.create("custom.op6", results=[i32]).results[0]
                    # CHECK: Value(%[[VAL6:.*]] = "custom.op6"() : () -> i32)
                    print(value6)

                    func.ReturnOp([])

                func.ReturnOp([])

        # CHECK: %[[VAL1]]
        print(value.get_name())
        # CHECK: %[[VAL2]]
        print(value2.get_name())
        # CHECK: %[[VAL3]]
        print(value3.get_name())
        # CHECK: %[[VAL4]]
        print(value4.get_name())

        # CHECK: %0
        print(value3.get_name(use_local_scope=True))
        # CHECK: %1
        print(value4.get_name(use_local_scope=True))

        # CHECK: %[[VAL5]]
        print(value5.get_name())
        # CHECK: %[[VAL6]]
        print(value6.get_name())

        # CHECK: %[[ARG0:.*]]
        print(entry_block1.arguments[0].get_name())
        # CHECK: %[[ARG1:.*]]
        print(entry_block1.arguments[1].get_name())

        # CHECK: %[[ARG2:.*]]
        print(entry_block2.arguments[0].get_name())
        # CHECK: %[[ARG3:.*]]
        print(entry_block2.arguments[1].get_name())

        # CHECK: module {
        # CHECK:   %[[VAL1]] = "custom.op1"() : () -> i32
        # CHECK:   %[[VAL2]] = "custom.op2"() : () -> i32
        # CHECK:   func.func @test(%[[ARG0]]: i32, %[[ARG1]]: i32) {
        # CHECK:     %[[VAL3]] = "custom.op3"() : () -> i32
        # CHECK:     %[[VAL4]] = "custom.op4"() : () -> i32
        # CHECK:     func @test(%[[ARG2]]: i32, %[[ARG3]]: i32) {
        # CHECK:       %[[VAL5]] = "custom.op5"() : () -> i32
        # CHECK:       %[[VAL6]] = "custom.op6"() : () -> i32
        # CHECK:       return
        # CHECK:     }
        # CHECK:     return
        # CHECK:   }
        # CHECK: }
        print(module)

        value2.owner.detach_from_parent()
        # CHECK: %0
        print(value2.get_name())



from lldbsuite.test import lldbinline
from lldbsuite.test import decorators

decor = [decorators.skipIf(compiler="clang", compiler_version=["<", "16.0"])]
lldbinline.MakeInlineTest(__file__, globals(), decor)



import lldb
from lldbsuite.test.lldbtest import *
from lldbsuite.test.decorators import *
import lldbsuite.test.lldbutil as lldbutil
import os


class TestMacCatalyst(TestBase):
    @skipIf(macos_version=["<", "10.15"])
    @skipUnlessDarwin
    @skipIfDarwinEmbedded
    def test_macabi(self):
        """Test the x86_64-apple-ios-macabi target linked against a macos dylib"""
        self.build()
        log = self.getBuildArtifact("packets.log")
        self.expect("log enable gdb-remote packets -f " + log)
        lldbutil.run_to_source_breakpoint(self, "break here", lldb.SBFileSpec("main.c"))
        self.expect(
            "image list -t -b",
            patterns=[self.getArchitecture() + r".*-apple-ios.*-macabi a\.out"],
        )
        self.expect("fr v s", substrs=["Hello macCatalyst"])
        self.expect("expression s", substrs=["Hello macCatalyst"])
        self.check_debugserver(log)

    def check_debugserver(self, log):
        """scan the debugserver packet log"""
        process_info = lldbutil.packetlog_get_process_info(log)
        self.assertIn("ostype", process_info)
        self.assertEqual(process_info["ostype"], "maccatalyst")

        aout_info = None
        dylib_info = lldbutil.packetlog_get_dylib_info(log)
        for image in dylib_info["images"]:
            if image["pathname"].endswith("a.out"):
                aout_info = image
        self.assertTrue(aout_info)
        self.assertEqual(aout_info["min_version_os_name"], "maccatalyst")



""" Testing debugging of a binary with "mixed" dwarf (with/without fission). """
import lldb
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil


class TestMixedDwarfBinary(TestBase):
    @no_debug_info_test  # Prevent the genaration of the dwarf version of this test
    @add_test_categories(["dwo"])
    @skipUnlessPlatform(["linux"])
    @skipIf(bugnumber="rdar://38550275")
    def test_mixed_dwarf(self):
        """Test that 'frame variable' works
        for the executable built from two source files compiled
        with/whithout -gsplit-dwarf correspondingly."""

        self.build()
        exe = self.getBuildArtifact("a.out")

        self.target = self.dbg.CreateTarget(exe)
        self.assertTrue(self.target, VALID_TARGET)

        main_bp = self.target.BreakpointCreateByName("g", "a.out")
        self.assertTrue(main_bp, VALID_BREAKPOINT)

        self.process = self.target.LaunchSimple(
            None, None, self.get_process_working_directory()
        )
        self.assertTrue(self.process, PROCESS_IS_VALID)

        # The stop reason of the thread should be breakpoint.
        self.assertState(
            self.process.GetState(), lldb.eStateStopped, STOPPED_DUE_TO_BREAKPOINT
        )

        frame = self.process.GetThreadAtIndex(0).GetFrameAtIndex(0)
        x = frame.FindVariable("x")
        self.assertTrue(x.IsValid(), "x is not valid")
        y = frame.FindVariable("y")
        self.assertTrue(y.IsValid(), "y is not valid")



from lldbsuite.test.lldbtest import *
from lldbsuite.test.decorators import *
import lldbsuite.test.lldbutil as lldbutil

class TestSwiftLateSwiftDylibClangDeps(TestBase):
    @skipIf(macos_version=["<", "14.0"])
    @skipUnlessDarwin
    @swiftTest
    @skipIfDarwinEmbedded 
    def test(self):
        """Test that the reflection metadata cache is invalidated
        when new DWARF debug info is available"""
        self.build()
        target, process, _, _ = lldbutil.run_to_source_breakpoint(
            self, "break here", lldb.SBFileSpec("loader.swift"))

        # Initialize SwiftASTContext before loading the dylib.
        self.runCmd("setting set symbols.swift-enable-ast-context false")
        self.expect("v fromClang",
                    substrs=["missing debug info", "FromClang"])

        bkpt = target.BreakpointCreateByLocation(
            lldb.SBFileSpec('dylib.swift'), 5)
        threads = lldbutil.continue_to_breakpoint(process, bkpt)

        self.expect("v x", substrs=['42'])
        self.expect("frame select 1")
        self.expect("v fromClang", substrs=['23'])



"""
Test trimming long progress report in tiny terminal windows
"""

import os
import pexpect
import tempfile
import re

import lldb
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test.lldbpexpect import PExpectTest


class TestTrimmedProgressReporting(PExpectTest):
    def do_test(self, term_width, pattern_list):
        self.build()
        # Start with a small window
        self.launch(use_colors=True)
        self.expect("set set show-progress true")
        self.expect(
            "set show show-progress", substrs=["show-progress (boolean) = true"]
        )
        self.expect("set set term-width " + str(term_width))
        self.expect(
            "set show term-width",
            substrs=["term-width (unsigned) = " + str(term_width)],
        )

        self.child.send("file " + self.getBuildArtifact("a.out") + "\n")
        self.child.expect(pattern_list)

    # PExpect uses many timeouts internally and doesn't play well
    # under ASAN on a loaded machine..
    @skipIfAsan
    @skipIfEditlineSupportMissing
    def test_trimmed_progress_message(self):
        self.do_test(19, ["Locating e...", "Parsing sy..."])

    # PExpect uses many timeouts internally and doesn't play well
    # under ASAN on a loaded machine..
    @skipIfAsan
    @skipIfEditlineSupportMissing
    def test_long_progress_message(self):
        self.do_test(
            80,
            [
                "Locating external symbol file",
                "Parsing symbol table",
            ],
        )



import lldb
from lldbsuite.test.lldbtest import *
from lldbsuite.test.decorators import *


class InvalidArgsLogTestCase(TestBase):
    @no_debug_info_test
    def test_enable_empty(self):
        self.expect(
            "log enable",
            error=True,
            substrs=[
                "error: log enable takes a log channel and one or more log types."
            ],
        )

    @no_debug_info_test
    def test_disable_empty(self):
        self.expect(
            "log disable",
            error=True,
            substrs=[
                "error: log disable takes a log channel and one or more log types."
            ],
        )

    @no_debug_info_test
    def test_enable_empty(self):
        invalid_path = os.path.join("this", "is", "not", "a", "valid", "path")
        self.expect(
            "log enable lldb all -f " + invalid_path,
            error=True,
            substrs=["Unable to open log file '" + invalid_path + "': ", "\n"],
        )



"""
Test that SBCompileUnit::FindLineEntryIndex works correctly.
"""

import lldb
import lldbsuite.test.lldbutil as lldbutil
from lldbsuite.test.lldbtest import *


class FindLineEntry(TestBase):
    def test_compile_unit_find_line_entry_index(self):
        """Test the CompileUnit LineEntryIndex lookup API"""
        self.build()
        exe = self.getBuildArtifact("a.out")
        self.target = self.dbg.CreateTarget(exe)
        self.assertTrue(self.target.IsValid(), "Target is not valid")

        self.file = lldb.SBFileSpec("main.c")
        sc_list = self.target.FindCompileUnits(self.file)
        self.assertEqual(len(sc_list), 1)
        cu = sc_list[0].GetCompileUnit()
        self.assertTrue(cu.IsValid(), "CompileUnit is not valid")

        # First look for valid line
        self.line = line_number("main.c", "int change_me")
        self.assertNotEqual(
            cu.FindLineEntryIndex(0, self.line, self.file),
            lldb.LLDB_INVALID_LINE_NUMBER,
        )

        # Then look for a line out of bound
        self.assertEqual(
            cu.FindLineEntryIndex(0, 42, self.file), lldb.LLDB_INVALID_LINE_NUMBER
        )



import lldb
from lldbsuite.test.lldbtest import *
from lldbsuite.test.decorators import *
import lldbsuite.test.lldbutil as lldbutil
import unittest2


class TestSwiftCTypeIvar(TestBase):
    @swiftTest
    @skipIf(setting=("symbols.use-swift-clangimporter", "false"))
    def test(self):
        """Test that the extra inhabitants are correctly computed for various
        kinds of Objective-C pointers, by using them in enums."""
        self.build()
        # Disable SwiftASTContext because we want to test we resolve the type
        # by looking up the clang type in debug info.
        self.runCmd("settings set symbols.swift-enable-ast-context false")
        lldbutil.run_to_source_breakpoint(
            self, "break here", lldb.SBFileSpec("main.swift")
        )
        # self.expect('v a', substrs=['asdf'])
        a = self.frame().FindVariable("a")
        lldbutil.check_variable(
            self,
            a.GetChildAtIndex(0),
            typename="Swift.Optional<Foo.BridgedPtr>",
            value="none",
        )

        b = self.frame().FindVariable("b")
        lldbutil.check_variable(
            self,
            a.GetChildAtIndex(0),
            typename="Swift.Optional<Foo.BridgedPtr>",
            value="none",
        )



import lldb
from lldbsuite.test.lldbtest import *
from lldbsuite.test.decorators import *
import lldbsuite.test.lldbutil as lldbutil
import unittest2


class TestSwiftOpaque(TestBase):

    @swiftTest
    def test(self):
        """Test opaque types in parameter positions"""         
        self.build()
        lldbutil.run_to_source_breakpoint(self, "break here",
                                          lldb.SBFileSpec("main.swift"))
        self.expect("v p", substrs=["C", "23"])




import lldb
from lldbsuite.test.lldbtest import *
from lldbsuite.test.decorators import *
import lldbsuite.test.lldbutil as lldbutil
import unittest2

class TestSwiftClangImporterExtraInhabitants(TestBase):
    @swiftTest
    @skipUnlessDarwin
    # Don't run ClangImporter tests if Clangimporter is disabled.
    @skipIf(setting=('symbols.use-swift-clangimporter', 'false'))
    def test(self):
        """Test that the extra inhabitants are correctly computed for various
           kinds of Objective-C pointers, by using them in enums."""
        self.build()
        lldbutil.run_to_source_breakpoint(self, 'break here',
                                          lldb.SBFileSpec('main.swift'))
        var = self.frame().FindVariable("mystruct")
        check = lldbutil.check_variable

        check(self, var.GetChildAtIndex(0), value="0")
        check(self, var.GetChildAtIndex(1),
              typename="Swift.Optional<Swift.OpaquePointer>",
              summary="nil")

        check(self, var.GetChildAtIndex(2), value="2")
        check(self, var.GetChildAtIndex(3),
              typename="Swift.Optional<Foo.BridgedPtr>",
              summary="nil")

        check(self, var.GetChildAtIndex(4), value="4")
        check(self, var.GetChildAtIndex(5),
              typename="Swift.Optional<Swift.AnyObject>",
              summary="nil")

        check(self, var.GetChildAtIndex(6), value="6")
        check(self, var.GetChildAtIndex(7),
              typename="Swift.Optional<Swift.UnsafeMutableRawPointer>",
              summary="nil")

        check(self, var.GetChildAtIndex(8), value="8")



"""
Test setting a breakpoint on an overloaded function by name.
"""

import re
import lldb
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil


class TestBreakpointOnOverload(TestBase):
    def check_breakpoint(self, name):
        bkpt = self.target.BreakpointCreateByName(name)
        self.assertEqual(bkpt.num_locations, 1, "Got one location")
        addr = bkpt.locations[0].GetAddress()
        self.assertTrue(addr.function.IsValid(), "Got a real function")
        # On Window, the name of the function includes the return value.
        # We still succeed in setting the breakpoint, but the resultant
        # name is not the same.
        # So just look for the name we used for the breakpoint in the
        # function name, rather than doing an equality check.
        self.assertIn(name, addr.function.name, "Got the right name")

    def test_break_on_overload(self):
        self.build()
        self.target = lldbutil.run_to_breakpoint_make_target(self)
        self.check_breakpoint("a_function(int)")
        self.check_breakpoint("a_function(double)")
        self.check_breakpoint("a_function(int, double)")
        self.check_breakpoint("a_function(double, int)")



"""
Test stepping into std::function
"""


import lldb
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil


class LibCxxFunctionSteppingIntoCallableTestCase(TestBase):
    NO_DEBUG_INFO_TESTCASE = True

    @skipIfDarwin
    @add_test_categories(["libc++"])
    def test(self):
        """Test that std::function as defined by libc++ is correctly printed by LLDB"""
        self.build()

        self.main_source = "main.cpp"
        self.main_source_spec = lldb.SBFileSpec(self.main_source)
        self.source_foo_line = line_number(self.main_source, "// Source foo start line")
        self.source_lambda_f2_line = line_number(
            self.main_source, "// Source lambda used by f2 start line"
        )
        self.source_lambda_f3_line = line_number(
            self.main_source, "// Source lambda used by f3 start line"
        )
        self.source_bar_operator_line = line_number(
            self.main_source, "// Source Bar::operator()() start line"
        )
        self.source_bar_add_num_line = line_number(
            self.main_source, "// Source Bar::add_num start line"
        )
        self.source_main_invoking_f1 = line_number(
            self.main_source, "// Source main invoking f1"
        )

        (target, process, thread, bkpt) = lldbutil.run_to_source_breakpoint(
            self, "// Set break point at this line.", self.main_source_spec
        )

        thread.StepInto()
        self.assertEqual(
            thread.GetFrameAtIndex(0).GetLineEntry().GetLine(),
            self.source_main_invoking_f1,
        )
        self.assertEqual(
            thread.GetFrameAtIndex(0).GetLineEntry().GetFileSpec().GetFilename(),
            self.main_source,
        )

        thread.StepInto()
        self.assertEqual(
            thread.GetFrameAtIndex(0).GetLineEntry().GetLine(), self.source_foo_line
        )
        self.assertEqual(
            thread.GetFrameAtIndex(0).GetLineEntry().GetFileSpec().GetFilename(),
            self.main_source,
        )
        process.Continue()

        thread.StepInto()
        self.assertEqual(
            thread.GetFrameAtIndex(0).GetLineEntry().GetLine(),
            self.source_lambda_f2_line,
        )
        self.assertEqual(
            thread.GetFrameAtIndex(0).GetLineEntry().GetFileSpec().GetFilename(),
            self.main_source,
        )
        process.Continue()

        thread.StepInto()
        self.assertEqual(
            thread.GetFrameAtIndex(0).GetLineEntry().GetLine(),
            self.source_lambda_f3_line,
        )
        self.assertEqual(
            thread.GetFrameAtIndex(0).GetLineEntry().GetFileSpec().GetFilename(),
            self.main_source,
        )
        process.Continue()

        # TODO reenable this case when std::function formatter supports
        # general callable object case.
        # thread.StepInto()
        # self.assertEqual( thread.GetFrameAtIndex(0).GetLineEntry().GetLine(), self.source_bar_operator_line ) ;
        # self.assertEqual( thread.GetFrameAtIndex(0).GetLineEntry().GetFileSpec().GetFilename(), self.main_source) ;
        # process.Continue()

        thread.StepInto()
        self.assertEqual(
            thread.GetFrameAtIndex(0).GetLineEntry().GetLine(),
            self.source_bar_add_num_line,
        )
        self.assertEqual(
            thread.GetFrameAtIndex(0).GetLineEntry().GetFileSpec().GetFilename(),
            self.main_source,
        )
        process.Continue()



"""
Make sure FindTypes finds struct types with the struct prefix.
"""


import lldb
import lldbsuite.test.lldbutil as lldbutil
from lldbsuite.test.lldbtest import *


class TestFindTypesOnStructType(TestBase):
    # If your test case doesn't stress debug info, then
    # set this to true.  That way it won't be run once for
    # each debug info format.
    NO_DEBUG_INFO_TESTCASE = True

    def test_find_types_struct_type(self):
        """Make sure FindTypes actually finds 'struct typename' not just 'typename'."""
        self.build()
        self.do_test()

    def do_test(self):
        """Make sure FindTypes actually finds 'struct typename' not just 'typename'."""
        exe = self.getBuildArtifact("a.out")

        # Create a target by the debugger.
        target = self.dbg.CreateTarget(exe)
        self.assertTrue(target, VALID_TARGET)

        # Make sure this works with struct
        type_list = target.FindTypes("struct mytype")
        self.assertEqual(
            type_list.GetSize(), 1, "Found one instance of the type with struct"
        )

        # Make sure this works without the struct:
        type_list = target.FindTypes("mytype")
        self.assertEqual(
            type_list.GetSize(), 1, "Found one instance of the type without struct"
        )

        # Make sure it works with union
        type_list = target.FindTypes("union myunion")
        self.assertEqual(
            type_list.GetSize(), 1, "Found one instance of the type with union"
        )

        # Make sure this works without the union:
        type_list = target.FindTypes("myunion")
        self.assertEqual(
            type_list.GetSize(), 1, "Found one instance of the type without union"
        )

        # Make sure it works with typedef
        type_list = target.FindTypes("typedef MyType")
        self.assertEqual(
            type_list.GetSize(), 1, "Found one instance of the type with typedef"
        )

        # Make sure this works without the typedef:
        type_list = target.FindTypes("MyType")
        self.assertEqual(
            type_list.GetSize(), 1, "Found one instance of the type without typedef"
        )



import lldb
from lldbsuite.test.lldbtest import *
from lldbsuite.test.decorators import *
import lldbsuite.test.lldbutil as lldbutil
import unittest2


class TestSwiftEmbeddedFrameVariable(TestBase):
    @skipUnlessDarwin
    @swiftTest
    def test(self):
        self.build()
        self.implementation()

    @skipUnlessDarwin
    @swiftTest
    def test_without_ast(self):
        """Run the test turning off instantion of  Swift AST contexts in order to ensure that all type information comes from DWARF"""
        self.build()
        self.runCmd("setting set symbols.swift-enable-ast-context false")
        self.implementation()

    def implementation(self):
        self.runCmd("setting set symbols.swift-enable-full-dwarf-debugging true")

        target, process, thread, _ = lldbutil.run_to_source_breakpoint(
            self, "break here", lldb.SBFileSpec("main.swift")
        )
        frame = thread.frames[0]
        self.assertTrue(frame, "Frame 0 is valid.")

        varB = frame.FindVariable("varB")
        field = varB.GetChildMemberWithName("a").GetChildMemberWithName("field")
        lldbutil.check_variable(self, field, False, value="4.5")
        b = varB.GetChildMemberWithName("b")
        lldbutil.check_variable(self, b, False, value="123456")

        tuple = frame.FindVariable("tuple")
        first = tuple.GetChildAtIndex(0)
        field = first.GetChildMemberWithName("field")
        lldbutil.check_variable(self, field, False, value="4.5")
        second = tuple.GetChildAtIndex(1)
        a = second.GetChildMemberWithName("a")
        field = a.GetChildMemberWithName("field")
        lldbutil.check_variable(self, field, False, value="4.5")
        b = second.GetChildMemberWithName("b")
        lldbutil.check_variable(self, b, False, value="123456")

        nonPayload1 = frame.FindVariable("nonPayload1")
        lldbutil.check_variable(self, nonPayload1, False, value="one")

        nonPayload2 = frame.FindVariable("nonPayload2")
        lldbutil.check_variable(self, nonPayload2, False, value="two")

        singlePayload = frame.FindVariable("singlePayload")
        payload = singlePayload.GetChildMemberWithName("payload")
        field = payload.GetChildMemberWithName("a").GetChildMemberWithName("field")
        lldbutil.check_variable(self, field, False, value="4.5")
        b = payload.GetChildMemberWithName("b")
        lldbutil.check_variable(self, b, False, value="123456")

        emptySinglePayload = frame.FindVariable("emptySinglePayload")
        lldbutil.check_variable(self, emptySinglePayload, False, value="nonPayloadTwo")

        smallMultipayloadEnum1 = frame.FindVariable("smallMultipayloadEnum1")
        one = smallMultipayloadEnum1.GetChildMemberWithName("one")
        lldbutil.check_variable(self, one, False, value="two")

        smallMultipayloadEnum2 = frame.FindVariable("smallMultipayloadEnum2")
        two = smallMultipayloadEnum2.GetChildMemberWithName("two")
        lldbutil.check_variable(self, two, False, value="one")

        bigMultipayloadEnum1 = frame.FindVariable("bigMultipayloadEnum1")
        one = bigMultipayloadEnum1.GetChildMemberWithName("one")
        first = one.GetChildAtIndex(0).GetChildMemberWithName("supField")
        second = one.GetChildAtIndex(1).GetChildMemberWithName("supField")
        third = one.GetChildAtIndex(2).GetChildMemberWithName("supField")
        lldbutil.check_variable(self, first, False, value="42")
        lldbutil.check_variable(self, second, False, value="43")
        lldbutil.check_variable(self, third, False, value="44")

        fullMultipayloadEnum1 = frame.FindVariable("fullMultipayloadEnum1")
        one = fullMultipayloadEnum1.GetChildMemberWithName("one")
        lldbutil.check_variable(self, one, False, value="120")

        fullMultipayloadEnum2 = frame.FindVariable("fullMultipayloadEnum2")
        two = fullMultipayloadEnum2.GetChildMemberWithName("two")
        lldbutil.check_variable(self, two, False, value="9.5")

        bigFullMultipayloadEnum1 = frame.FindVariable("bigFullMultipayloadEnum1")
        one = bigFullMultipayloadEnum1.GetChildMemberWithName("one")
        first = one.GetChildAtIndex(0)
        second = one.GetChildAtIndex(1)
        lldbutil.check_variable(self, first, False, value="209")
        lldbutil.check_variable(self, second, False, value="315")

        bigFullMultipayloadEnum2 = frame.FindVariable("bigFullMultipayloadEnum2")
        two = bigFullMultipayloadEnum2.GetChildMemberWithName("two")
        first = two.GetChildAtIndex(0)
        second = two.GetChildAtIndex(1)
        lldbutil.check_variable(self, first, False, value="452.5")
        lldbutil.check_variable(self, second, False, value="753.5")

        sup = frame.FindVariable("sup")
        supField = sup.GetChildMemberWithName("supField")
        lldbutil.check_variable(self, supField, False, value="42")

        sub = frame.FindVariable("sub")
        supField = sub.GetChildMemberWithName("supField")
        lldbutil.check_variable(self, supField, False, value="42")
        subField = sub.GetChildMemberWithName("subField")
        a = subField.GetChildMemberWithName("a")
        field = a.GetChildMemberWithName("field")
        lldbutil.check_variable(self, field, False, value="4.5")
        b = subField.GetChildMemberWithName("b")
        lldbutil.check_variable(self, b, False, value="123456")

        subSub = frame.FindVariable("subSub")
        supField = subSub.GetChildMemberWithName("supField")
        lldbutil.check_variable(self, supField, False, value="42")
        subField = subSub.GetChildMemberWithName("subField")
        a = subField.GetChildMemberWithName("a")
        field = a.GetChildMemberWithName("field")
        lldbutil.check_variable(self, field, False, value="4.5")
        b = subField.GetChildMemberWithName("b")
        lldbutil.check_variable(self, b, False, value="123456")

        subSubField = subSub.GetChildMemberWithName(
            "subSubField"
        ).GetChildMemberWithName("field")
        lldbutil.check_variable(self, subSubField, False, value="4.5")

        gsp = frame.FindVariable("gsp")
        t = gsp.GetChildMemberWithName("t")
        lldbutil.check_variable(self, t, False, value="42")
        u = gsp.GetChildMemberWithName("u")
        lldbutil.check_variable(self, u, False, value="94.5")

        gsp2 = frame.FindVariable("gsp2")
        t = gsp2.GetChildMemberWithName("t")
        supField = t.GetChildMemberWithName("supField")
        lldbutil.check_variable(self, supField, False, value="42")
        u = gsp2.GetChildMemberWithName("u")
        a = u.GetChildMemberWithName("a")
        field = a.GetChildMemberWithName("field")
        lldbutil.check_variable(self, field, False, value="4.5")
        b = u.GetChildMemberWithName("b")
        lldbutil.check_variable(self, b, False, value="123456")

        gsp3 = frame.FindVariable("gsp3")
        t = gsp3.GetChildMemberWithName("t")
        one = t.GetChildMemberWithName("one")
        first = one.GetChildAtIndex(0)
        second = one.GetChildAtIndex(1)
        lldbutil.check_variable(self, first, False, value="209")
        lldbutil.check_variable(self, second, False, value="315")
        u = gsp3.GetChildMemberWithName("u")
        two = u.GetChildMemberWithName("two")
        lldbutil.check_variable(self, two, False, value="one")

        gcp = frame.FindVariable("gcp")
        t = gcp.GetChildMemberWithName("t")
        lldbutil.check_variable(self, t, False, value="55.5")
        u = gcp.GetChildMemberWithName("u")
        lldbutil.check_variable(self, u, False, value="9348")

        either = frame.FindVariable("either")
        left = either.GetChildMemberWithName("left")
        lldbutil.check_variable(self, left, False, value="1234")

        either2 = frame.FindVariable("either2")
        right = either2.GetChildMemberWithName("right")
        t = right.GetChildMemberWithName("t")
        one = t.GetChildMemberWithName("one")
        first = one.GetChildAtIndex(0)
        second = one.GetChildAtIndex(1)
        lldbutil.check_variable(self, first, False, value="209")
        lldbutil.check_variable(self, second, False, value="315")
        u = right.GetChildMemberWithName("u")
        two = u.GetChildMemberWithName("two")
        lldbutil.check_variable(self, two, False, value='one')

        inner = frame.FindVariable("inner")
        value = inner.GetChildMemberWithName("value")
        lldbutil.check_variable(self, value, False, value='99')

        innerer = frame.FindVariable("innerer")
        innererValue = innerer.GetChildMemberWithName("innererValue")
        lldbutil.check_variable(self, innererValue, False, value='101')

        privateType = frame.FindVariable("privateType")
        privateField = privateType.GetChildMemberWithName("privateField")
        lldbutil.check_variable(self, privateField, False, value='100')

        specializedInner = frame.FindVariable("specializedInner")
        t = specializedInner.GetChildMemberWithName("t")
        lldbutil.check_variable(self, t, False, value='837')

        genericInner = frame.FindVariable("genericInner")
        t = genericInner.GetChildMemberWithName("t")
        lldbutil.check_variable(self, t, False, value='647')
        u = genericInner.GetChildMemberWithName("u")
        lldbutil.check_variable(self, u, False, value='674.5')

        functionType = frame.FindVariable("functionType")
        funcField = functionType.GetChildMemberWithName("funcField")
        lldbutil.check_variable(self, funcField, False, value='67')

        innerFunctionType = frame.FindVariable("innerFunctionType")
        innerFuncField = innerFunctionType.GetChildMemberWithName("innerFuncField")
        lldbutil.check_variable(self, innerFuncField, False, value='8479')



"""Test that the 'add-dsym', aka 'target symbols add', command informs the user about success or failure."""


import os
import time
import lldb
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil


@skipUnlessDarwin
class AddDsymCommandCase(TestBase):
    def setUp(self):
        TestBase.setUp(self)
        self.template = "main.cpp.template"
        self.source = "main.cpp"
        self.teardown_hook_added = False

    @no_debug_info_test
    def test_add_dsym_command_with_error(self):
        """Test that the 'add-dsym' command informs the user about failures."""

        # Call the program generator to produce main.cpp, version 1.
        self.generate_main_cpp(version=1)
        self.build(debug_info="dsym")

        # Insert some delay and then call the program generator to produce
        # main.cpp, version 2.
        time.sleep(5)
        self.generate_main_cpp(version=101)
        # Now call make again, but this time don't generate the dSYM.
        self.build(debug_info="dwarf")

        self.exe_name = "a.out"
        self.do_add_dsym_with_error(self.exe_name)

    @no_debug_info_test
    def test_add_dsym_command_with_success(self):
        """Test that the 'add-dsym' command informs the user about success."""

        # Call the program generator to produce main.cpp, version 1.
        self.generate_main_cpp(version=1)
        self.build(debug_info="dsym")

        self.exe_name = "a.out"
        self.do_add_dsym_with_success(self.exe_name)

    @no_debug_info_test
    def test_add_dsym_with_dSYM_bundle(self):
        """Test that the 'add-dsym' command informs the user about success."""

        # Call the program generator to produce main.cpp, version 1.
        self.generate_main_cpp(version=1)
        self.build(debug_info="dsym")

        self.exe_name = "a.out"
        self.do_add_dsym_with_dSYM_bundle(self.exe_name)

    def generate_main_cpp(self, version=0):
        """Generate main.cpp from main.cpp.template."""
        temp = os.path.join(self.getSourceDir(), self.template)
        with open(temp, "r") as f:
            content = f.read()

        new_content = content.replace(
            "%ADD_EXTRA_CODE%", 'printf("This is version %d\\n");' % version
        )
        src = os.path.join(self.getBuildDir(), self.source)
        with open(src, "w") as f:
            f.write(new_content)

        # The main.cpp has been generated, add a teardown hook to remove it.
        if not self.teardown_hook_added:
            self.addTearDownHook(lambda: os.remove(src))
            self.teardown_hook_added = True

    def do_add_dsym_with_error(self, exe_name):
        """Test that the 'add-dsym' command informs the user about failures."""
        exe_path = self.getBuildArtifact(exe_name)
        self.runCmd("file " + exe_path, CURRENT_EXECUTABLE_SET)

        wrong_path = os.path.join(self.getBuildDir(), "%s.dSYM" % exe_name, "Contents")
        self.expect(
            "add-dsym " + wrong_path, error=True, substrs=["invalid module path"]
        )

        right_path = os.path.join(
            self.getBuildDir(),
            "%s.dSYM" % exe_path,
            "Contents",
            "Resources",
            "DWARF",
            exe_name,
        )
        self.expect(
            "add-dsym " + right_path,
            error=True,
            substrs=["symbol file", "does not match"],
        )

    def do_add_dsym_with_success(self, exe_name):
        """Test that the 'add-dsym' command informs the user about success."""
        exe_path = self.getBuildArtifact(exe_name)
        self.runCmd("file " + exe_path, CURRENT_EXECUTABLE_SET)

        # This time, the UUID should match and we expect some feedback from
        # lldb.
        right_path = os.path.join(
            self.getBuildDir(),
            "%s.dSYM" % exe_path,
            "Contents",
            "Resources",
            "DWARF",
            exe_name,
        )
        self.expect(
            "add-dsym " + right_path, substrs=["symbol file", "has been added to"]
        )

    def do_add_dsym_with_dSYM_bundle(self, exe_name):
        """Test that the 'add-dsym' command informs the user about success when loading files in bundles."""
        exe_path = self.getBuildArtifact(exe_name)
        self.runCmd("file " + exe_path, CURRENT_EXECUTABLE_SET)

        # This time, the UUID should be found inside the bundle
        right_path = "%s.dSYM" % exe_path
        self.expect(
            "add-dsym " + right_path, substrs=["symbol file", "has been added to"]
        )



import lldb
from lldbsuite.test.lldbtest import *
from lldbsuite.test.decorators import *
import lldbsuite.test.lldbutil as lldbutil
import unittest2

class TestSwiftExprImport(TestBase):
    # Don't run ClangImporter tests if Clangimporter is disabled.
    @skipIf(setting=('symbols.use-swift-clangimporter', 'false'))
    @skipIf(setting=('symbols.swift-precise-compiler-invocation', 'true'))
    @swiftTest
    def test(self):
        """Test error handling if the expression evaluator
           encounters a Clang import failure.
        """
        self.build()

        lldbutil.run_to_source_breakpoint(self, "break here",
                                          lldb.SBFileSpec('main.swift'))
        self.expect("expr -- import A", error=True,
                    substrs=['SYNTAX_ERROR',
                             'could', 'not', 'build', 'module'])




"""
Test that Swift types are displayed correctly in C++
"""
from lldbsuite.test.lldbtest import *
from lldbsuite.test.decorators import *


class TestSwiftBackwardInteropFormatSwiftStdlibTypes(TestBase):
    def setup(self, bkpt_str): 
        self.build()
        
        _, _, _, _= lldbutil.run_to_source_breakpoint(
            self, bkpt_str, lldb.SBFileSpec('main.cpp'))


    @swiftTest
    def test_array(self):
        self.setup('break here for array')
        self.expect('v array', substrs=['Swift.Array<a.SwiftClass>',
            '[0]', 'str = "Hello from the Swift class!"', 
            '[1]', 'str = "Hello from the Swift class!"',])

    @swiftTest
    def test_array_of_ints(self):
        self.setup('break here for array of ints')

        self.expect('v array', substrs=['Swift.Array<Swift.Int32>', '1', '2', '3', '4'])

    @swiftTest
    def test_optional(self):
        self.setup('break here for optional')

        self.expect('v optional', substrs=['Swift.Optional<a.SwiftClass>', 
            'str = "Hello from the Swift class!"'])

    @swiftTest
    def test_optional_primitive(self):
        self.setup('break here for optional primitive')

        self.expect('v optional', substrs=['Swift.Optional<Swift.Double>', 
            '4.2'])

    @swiftTest
    def test_string(self):
        self.setup('break here for string')

        self.expect('v string', substrs=['"Hello from Swift!"'])




"""Test function call thread safety."""


import lldb
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil


class TestSafeFuncCalls(TestBase):
    @skipUnlessDarwin
    @add_test_categories(["pyapi"])
    def test_with_python_api(self):
        """Test function call thread safety."""
        self.build()
        exe = self.getBuildArtifact("a.out")

        target = self.dbg.CreateTarget(exe)
        self.assertTrue(target, VALID_TARGET)
        self.main_source_spec = lldb.SBFileSpec("main.c")
        break1 = target.BreakpointCreateByName("stopper", "a.out")
        self.assertTrue(break1, VALID_BREAKPOINT)
        process = target.LaunchSimple(None, None, self.get_process_working_directory())
        self.assertTrue(process, PROCESS_IS_VALID)
        threads = lldbutil.get_threads_stopped_at_breakpoint(process, break1)
        self.assertEqual(len(threads), 1, "Failed to stop at breakpoint 1.")

        self.assertEqual(
            process.GetNumThreads(),
            2,
            "Check that the process has two threads when sitting at the stopper() breakpoint",
        )

        main_thread = lldb.SBThread()
        select_thread = lldb.SBThread()
        for idx in range(0, process.GetNumThreads()):
            t = process.GetThreadAtIndex(idx)
            if t.GetName() == "main thread":
                main_thread = t
            if t.GetName() == "select thread":
                select_thread = t

        self.assertTrue(
            main_thread.IsValid() and select_thread.IsValid(),
            "Got both expected threads",
        )

        self.assertTrue(
            main_thread.SafeToCallFunctions(),
            "It is safe to call functions on the main thread",
        )
        self.assertEqual(
            select_thread.SafeToCallFunctions(),
            False,
            "It is not safe to call functions on the select thread",
        )



"""
Test swift Class types
"""
import lldb
from lldbsuite.test.lldbtest import *
from lldbsuite.test.decorators import *
import lldbsuite.test.lldbutil as lldbutil
import os
import unittest2


class TestSwiftActorTypes(TestBase):
    @swiftTest
    def test_swift_class_types(self):
        """Test swift Actor types"""
        self.build()
        _, _, _, _= lldbutil.run_to_source_breakpoint(
            self, 'Set breakpoint here', lldb.SBFileSpec('main.swift'))

        self.expect('v actor', substrs=['Actor', 'str = "Hello"'])
        self.expect('expr actor', substrs=['Actor', 'str = "Hello"'])




"""
Fuzz tests an object after the default construction to make sure it does not crash lldb.
"""

import lldb


def fuzz_obj(obj):
    obj.Append(lldb.SBValue())
    obj.GetSize()
    obj.GetValueAtIndex(100)
    obj.FindValueObjectByUID(200)
    for val in obj:
        s = str(val)



# RUN: %PYTHON %s | FileCheck %s

import gc

from mlir.ir import *


def run(f):
    print("\nTEST:", f.__name__)
    f()
    gc.collect()
    assert Context._get_live_count() == 0


def add_dummy_value():
    return Operation.create(
        "custom.value", results=[IntegerType.get_signless(32)]
    ).result


def testOdsBuildDefaultImplicitRegions():
    class TestFixedRegionsOp(OpView):
        OPERATION_NAME = "custom.test_op"
        _ODS_REGIONS = (2, True)

    class TestVariadicRegionsOp(OpView):
        OPERATION_NAME = "custom.test_any_regions_op"
        _ODS_REGIONS = (2, False)

    with Context() as ctx, Location.unknown():
        ctx.allow_unregistered_dialects = True
        m = Module.create()
        with InsertionPoint(m.body):
            op = TestFixedRegionsOp.build_generic(results=[], operands=[])
            # CHECK: NUM_REGIONS: 2
            print(f"NUM_REGIONS: {len(op.regions)}")
            # Including a regions= that matches should be fine.
            op = TestFixedRegionsOp.build_generic(results=[], operands=[], regions=2)
            print(f"NUM_REGIONS: {len(op.regions)}")
            # Reject greater than.
            try:
                op = TestFixedRegionsOp.build_generic(
                    results=[], operands=[], regions=3
                )
            except ValueError as e:
                # CHECK: ERROR:Operation "custom.test_op" requires a maximum of 2 regions but was built with regions=3
                print(f"ERROR:{e}")
            # Reject less than.
            try:
                op = TestFixedRegionsOp.build_generic(
                    results=[], operands=[], regions=1
                )
            except ValueError as e:
                # CHECK: ERROR:Operation "custom.test_op" requires a minimum of 2 regions but was built with regions=1
                print(f"ERROR:{e}")

            # If no regions specified for a variadic region op, build the minimum.
            op = TestVariadicRegionsOp.build_generic(results=[], operands=[])
            # CHECK: DEFAULT_NUM_REGIONS: 2
            print(f"DEFAULT_NUM_REGIONS: {len(op.regions)}")
            # Should also accept an explicit regions= that matches the minimum.
            op = TestVariadicRegionsOp.build_generic(results=[], operands=[], regions=2)
            # CHECK: EQ_NUM_REGIONS: 2
            print(f"EQ_NUM_REGIONS: {len(op.regions)}")
            # And accept greater than minimum.
            # Should also accept an explicit regions= that matches the minimum.
            op = TestVariadicRegionsOp.build_generic(results=[], operands=[], regions=3)
            # CHECK: GT_NUM_REGIONS: 3
            print(f"GT_NUM_REGIONS: {len(op.regions)}")
            # Should reject less than minimum.
            try:
                op = TestVariadicRegionsOp.build_generic(
                    results=[], operands=[], regions=1
                )
            except ValueError as e:
                # CHECK: ERROR:Operation "custom.test_any_regions_op" requires a minimum of 2 regions but was built with regions=1
                print(f"ERROR:{e}")


run(testOdsBuildDefaultImplicitRegions)


def testOdsBuildDefaultNonVariadic():
    class TestOp(OpView):
        OPERATION_NAME = "custom.test_op"

    with Context() as ctx, Location.unknown():
        ctx.allow_unregistered_dialects = True
        m = Module.create()
        with InsertionPoint(m.body):
            v0 = add_dummy_value()
            v1 = add_dummy_value()
            t0 = IntegerType.get_signless(8)
            t1 = IntegerType.get_signless(16)
            op = TestOp.build_generic(results=[t0, t1], operands=[v0, v1])
            # CHECK: %[[V0:.+]] = "custom.value"
            # CHECK: %[[V1:.+]] = "custom.value"
            # CHECK: "custom.test_op"(%[[V0]], %[[V1]])
            # CHECK-NOT: operandSegmentSizes
            # CHECK-NOT: resultSegmentSizes
            # CHECK-SAME: : (i32, i32) -> (i8, i16)
            print(m)


run(testOdsBuildDefaultNonVariadic)


def testOdsBuildDefaultSizedVariadic():
    class TestOp(OpView):
        OPERATION_NAME = "custom.test_op"
        _ODS_OPERAND_SEGMENTS = [1, -1, 0]
        _ODS_RESULT_SEGMENTS = [-1, 0, 1]

    with Context() as ctx, Location.unknown():
        ctx.allow_unregistered_dialects = True
        m = Module.create()
        with InsertionPoint(m.body):
            v0 = add_dummy_value()
            v1 = add_dummy_value()
            v2 = add_dummy_value()
            v3 = add_dummy_value()
            t0 = IntegerType.get_signless(8)
            t1 = IntegerType.get_signless(16)
            t2 = IntegerType.get_signless(32)
            t3 = IntegerType.get_signless(64)
            # CHECK: %[[V0:.+]] = "custom.value"
            # CHECK: %[[V1:.+]] = "custom.value"
            # CHECK: %[[V2:.+]] = "custom.value"
            # CHECK: %[[V3:.+]] = "custom.value"
            # CHECK: "custom.test_op"(%[[V0]], %[[V1]], %[[V2]], %[[V3]])
            # CHECK-SAME: operandSegmentSizes = array<i32: 1, 2, 1>
            # CHECK-SAME: resultSegmentSizes = array<i32: 2, 1, 1>
            # CHECK-SAME: : (i32, i32, i32, i32) -> (i8, i16, i32, i64)
            op = TestOp.build_generic(
                results=[[t0, t1], t2, t3], operands=[v0, [v1, v2], v3]
            )

            # Now test with optional omitted.
            # CHECK: "custom.test_op"(%[[V0]])
            # CHECK-SAME: operandSegmentSizes = array<i32: 1, 0, 0>
            # CHECK-SAME: resultSegmentSizes = array<i32: 0, 0, 1>
            # CHECK-SAME: (i32) -> i64
            op = TestOp.build_generic(
                results=[None, None, t3], operands=[v0, None, None]
            )
            print(m)

            # And verify that errors are raised for None in a required operand.
            try:
                op = TestOp.build_generic(
                    results=[None, None, t3], operands=[None, None, None]
                )
            except ValueError as e:
                # CHECK: OPERAND_CAST_ERROR:Operand 0 of operation "custom.test_op" must be a Value (was None and operand is not optional)
                print(f"OPERAND_CAST_ERROR:{e}")

            # And verify that errors are raised for None in a required result.
            try:
                op = TestOp.build_generic(
                    results=[None, None, None], operands=[v0, None, None]
                )
            except ValueError as e:
                # CHECK: RESULT_CAST_ERROR:Result 2 of operation "custom.test_op" must be a Type (was None and result is not optional)
                print(f"RESULT_CAST_ERROR:{e}")

            # Variadic lists with None elements should reject.
            try:
                op = TestOp.build_generic(
                    results=[None, None, t3], operands=[v0, [None], None]
                )
            except ValueError as e:
                # CHECK: OPERAND_LIST_CAST_ERROR:Operand 1 of operation "custom.test_op" must be a Sequence of Values (contained a None item)
                print(f"OPERAND_LIST_CAST_ERROR:{e}")
            try:
                op = TestOp.build_generic(
                    results=[[None], None, t3], operands=[v0, None, None]
                )
            except ValueError as e:
                # CHECK: RESULT_LIST_CAST_ERROR:Result 0 of operation "custom.test_op" must be a Sequence of Types (contained a None item)
                print(f"RESULT_LIST_CAST_ERROR:{e}")


run(testOdsBuildDefaultSizedVariadic)


def testOdsBuildDefaultCastError():
    class TestOp(OpView):
        OPERATION_NAME = "custom.test_op"

    with Context() as ctx, Location.unknown():
        ctx.allow_unregistered_dialects = True
        m = Module.create()
        with InsertionPoint(m.body):
            v0 = add_dummy_value()
            v1 = add_dummy_value()
            t0 = IntegerType.get_signless(8)
            t1 = IntegerType.get_signless(16)
            try:
                op = TestOp.build_generic(results=[t0, t1], operands=[None, v1])
            except ValueError as e:
                # CHECK: ERROR: Operand 0 of operation "custom.test_op" must be a Value
                print(f"ERROR: {e}")
            try:
                op = TestOp.build_generic(results=[t0, None], operands=[v0, v1])
            except ValueError as e:
                # CHECK: Result 1 of operation "custom.test_op" must be a Type
                print(f"ERROR: {e}")


run(testOdsBuildDefaultCastError)



#!/bin/python3

# This file is licensed under the Apache License v2.0 with LLVM Exceptions.
# See https://llvm.org/LICENSE.txt for license information.
# SPDX-License-Identifier: Apache-2.0 WITH LLVM-exception
"""Overlays two directories into a target directory using symlinks.

Tries to minimize the number of symlinks created (that is, does not symlink
every single file). Symlinks every file in the overlay directory. Only symlinks
individual files in the source directory if their parent directory is also
contained in the overlay directory tree.
"""

import argparse
import errno
import os
import sys


def _check_python_version():
    if sys.version_info[0] < 3:
        raise RuntimeError(
            "Must be invoked with a python 3 interpreter but was %s" % sys.executable
        )


def _check_dir_exists(path):
    if not os.path.isdir(path):
        raise OSError(errno.ENOENT, os.strerror(errno.ENOENT), path)


def parse_arguments():
    parser = argparse.ArgumentParser(
        description="""
    Overlays two directories into a target directory using symlinks.

    Tries to minimize the number of symlinks created (that is, does not symlink
    every single file). Symlinks every file in the overlay directory. Only
    symlinks individual files in the source directory if their parent directory
    is also contained in the overlay directory tree.
    """
    )
    parser.add_argument(
        "--src",
        required=True,
        help="Directory that contains most of the content to symlink.",
    )
    parser.add_argument(
        "--overlay",
        required=True,
        help="Directory to overlay on top of the source directory.",
    )
    parser.add_argument(
        "--target",
        required=True,
        help="Directory in which to place the fused symlink directories.",
    )

    args = parser.parse_args()

    _check_dir_exists(args.target)
    _check_dir_exists(args.overlay)
    _check_dir_exists(args.src)

    return args


def _symlink_abs(from_path, to_path):
    os.symlink(os.path.abspath(from_path), os.path.abspath(to_path))


def main(args):
    for root, dirs, files in os.walk(args.overlay):
        # We could do something more intelligent here and only symlink individual
        # files if the directory is present in both overlay and src. This could also
        # be generalized to an arbitrary number of directories without any
        # "src/overlay" distinction. In the current use case we only have two and
        # the overlay directory is always small, so putting that off for now.
        rel_root = os.path.relpath(root, start=args.overlay)
        if rel_root != ".":
            os.mkdir(os.path.join(args.target, rel_root))

        for file in files:
            relpath = os.path.join(rel_root, file)
            _symlink_abs(
                os.path.join(args.overlay, relpath), os.path.join(args.target, relpath)
            )

        for src_entry in os.listdir(os.path.join(args.src, rel_root)):
            if src_entry not in dirs:
                relpath = os.path.join(rel_root, src_entry)
                _symlink_abs(
                    os.path.join(args.src, relpath), os.path.join(args.target, relpath)
                )


if __name__ == "__main__":
    _check_python_version()
    main(parse_arguments())



# TestSwiftExpressionsInClassFunctions.py
#
# This source file is part of the Swift.org open source project
#
# Copyright (c) 2014 - 2016 Apple Inc. and the Swift project authors
# Licensed under Apache License v2.0 with Runtime Library Exception
#
# See https://swift.org/LICENSE.txt for license information
# See https://swift.org/CONTRIBUTORS.txt for the list of Swift project authors
#
# ------------------------------------------------------------------------------
"""
Test expressions in the context of class functions
"""
import lldb
from lldbsuite.test.lldbtest import *
from lldbsuite.test.decorators import *
import lldbsuite.test.lldbutil as lldbutil
import os
import unittest2


class TestSwiftExpressionsInClassFunctions(TestBase):
    @swiftTest
    def test_expressions_in_class_functions(self):
        """Test expressions in class func contexts"""
        self.build()
        target = self.dbg.CreateTarget(self.getBuildArtifact())
        self.assertTrue(target, VALID_TARGET)

        # Set the breakpoints
        breakpoints = [None]
        for i in range(1, 8):
            breakpoints.append(
                target.BreakpointCreateBySourceRegex(
                    "breakpoint " + str(i), lldb.SBFileSpec('main.swift')))
            self.assertTrue(
                breakpoints[i].GetNumLocations() > 0,
                "Didn't get valid breakpoint for %s" % (str(i)))

        # Launch the process, and do not stop at the entry point.
        process = target.LaunchSimple(None, None, os.getcwd())
        self.assertTrue(process, PROCESS_IS_VALID)

        # Check each context
        for i in range(1, 8):
            # Frame #0 should be at our breakpoint.
            threads = lldbutil.get_threads_stopped_at_breakpoint(
                process, breakpoints[i])

            self.assertTrue(len(threads) == 1)
            lldbutil.check_expression(self, self.frame(), "i", str(i), False)
            if i == 6:
              lldbutil.check_expression(self, self.frame(), "self", "a.H<Int>")
              frame = threads[0].GetFrameAtIndex(0)
              lldbutil.check_variable(self, frame.FindVariable("self"),
                                      # FIXME: This should be '@thick a.H<Swift.Int>.Type'
                                      # but valobj.GetDynamicValue(lldb.eDynamicCanRunTarget)
                                      # doesn't seem to do its job.
                                      # rdar://problem/69889462
                                      typename='@thin a.H<τ_0_0>.Type')

            self.runCmd("continue")




#  Part of the LLVM Project, under the Apache License v2.0 with LLVM Exceptions.
#  See https://llvm.org/LICENSE.txt for license information.
#  SPDX-License-Identifier: Apache-2.0 WITH LLVM-exception

#  This file contains the sparse compiler class. It is copied from
#  test/Integration/Dialect/SparseTensor/python/ until we have a better
#  solution.

from mlir import execution_engine
from mlir import ir
from mlir import passmanager
from typing import Sequence


class SparseCompiler:
    """Sparse compiler class for compiling and building MLIR modules."""

    def __init__(self, options: str, opt_level: int, shared_libs: Sequence[str]):
        pipeline = f"builtin.module(sparse-compiler{{{options} reassociate-fp-reductions=1 enable-index-optimizations=1}})"
        self.pipeline = pipeline
        self.opt_level = opt_level
        self.shared_libs = shared_libs

    def __call__(self, module: ir.Module):
        """Convenience application method."""
        self.compile(module)

    def compile(self, module: ir.Module):
        """Compiles the module by invoking the sparse copmiler pipeline."""
        passmanager.PassManager.parse(self.pipeline).run(module.operation)

    def jit(self, module: ir.Module) -> execution_engine.ExecutionEngine:
        """Wraps the module in a JIT execution engine."""
        return execution_engine.ExecutionEngine(
            module, opt_level=self.opt_level, shared_libs=self.shared_libs
        )

    def compile_and_jit(self, module: ir.Module) -> execution_engine.ExecutionEngine:
        """Compiles and jits the module."""
        self.compile(module)
        return self.jit(module)



"""Test stepping over vrs. hitting breakpoints & subsequent stepping in various forms."""


import lldb
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil


class StepUntilTestCase(TestBase):
    def setUp(self):
        # Call super's setUp().
        TestBase.setUp(self)
        # Find the line numbers that we will step to in main:
        self.main_source = "main.c"
        self.less_than_two = line_number("main.c", "Less than 2")
        self.greater_than_two = line_number("main.c", "Greater than or equal to 2.")
        self.back_out_in_main = line_number("main.c", "Back out in main")

    def common_setup(self, args):
        self.build()
        exe = self.getBuildArtifact("a.out")

        target = self.dbg.CreateTarget(exe)
        self.assertTrue(target, VALID_TARGET)

        main_source_spec = lldb.SBFileSpec(self.main_source)
        break_before = target.BreakpointCreateBySourceRegex(
            "At the start", main_source_spec
        )
        self.assertTrue(break_before, VALID_BREAKPOINT)

        # Now launch the process, and do not stop at entry point.
        process = target.LaunchSimple(args, None, self.get_process_working_directory())

        self.assertTrue(process, PROCESS_IS_VALID)

        # The stop reason of the thread should be breakpoint.
        threads = lldbutil.get_threads_stopped_at_breakpoint(process, break_before)

        if len(threads) != 1:
            self.fail("Failed to stop at first breakpoint in main.")

        thread = threads[0]
        return thread

    def do_until(self, args, until_lines, expected_linenum):
        thread = self.common_setup(args)

        cmd_interp = self.dbg.GetCommandInterpreter()
        ret_obj = lldb.SBCommandReturnObject()

        cmd_line = "thread until"
        for line_num in until_lines:
            cmd_line += " %d" % (line_num)

        cmd_interp.HandleCommand(cmd_line, ret_obj)
        self.assertTrue(
            ret_obj.Succeeded(), "'%s' failed: %s." % (cmd_line, ret_obj.GetError())
        )

        frame = thread.frames[0]
        line = frame.GetLineEntry().GetLine()
        self.assertEqual(
            line, expected_linenum, "Did not get the expected stop line number"
        )

    def test_hitting_one(self):
        """Test thread step until - targeting one line and hitting it."""
        self.do_until(None, [self.less_than_two], self.less_than_two)

    def test_targetting_two_hitting_first(self):
        """Test thread step until - targeting two lines and hitting one."""
        self.do_until(
            ["foo", "bar", "baz"],
            [self.less_than_two, self.greater_than_two],
            self.greater_than_two,
        )

    def test_targetting_two_hitting_second(self):
        """Test thread step until - targeting two lines and hitting the other one."""
        self.do_until(
            None, [self.less_than_two, self.greater_than_two], self.less_than_two
        )

    def test_missing_one(self):
        """Test thread step until - targeting one line and missing it by stepping out to call site"""
        self.do_until(
            ["foo", "bar", "baz"], [self.less_than_two], self.back_out_in_main
        )



import lldb
from lldbsuite.test.decorators import *
import lldbsuite.test.lldbtest as lldbtest
import lldbsuite.test.lldbutil as lldbutil
import os
import unittest2


class TestSwiftXcodeSDK(lldbtest.TestBase):

    mydir = lldbtest.TestBase.compute_mydir(__file__)
    NO_DEBUG_INFO_TESTCASE = True

    def check_log(self, log, expected_path):
        import io
        logfile = io.open(log, "r", encoding='utf-8')
        in_expr_log = 0
        found = 0
        for line in logfile:
            if line.startswith(" SwiftASTContextForExpressions::LogConfiguration(SwiftASTContext"):
                in_expr_log += 1
            if in_expr_log and "SDK path" in line and expected_path in line:
                found += 1
        self.assertEqual(in_expr_log, 1)
        self.assertEqual(found, 1)

    @swiftTest
    @skipUnlessDarwin
    @skipIfDarwinEmbedded
    def test_decode(self):
        """Test that we can detect an Xcode SDK from the DW_AT_LLVM_sdk attribute."""
        self.build()
        log = self.getBuildArtifact("types.log")
        self.expect("log enable lldb types -f " + log)

        lldbutil.run_to_name_breakpoint(self, 'main')

        self.expect("expression 1")
        self.check_log(log, "MacOSX")

    @swiftTest
    @skipUnlessDarwin
    @skipIfDarwinEmbedded
    @apple_simulator_test('iphone')
    # FIXME: This test depends on https://reviews.llvm.org/D81980.
    @expectedFailureAll(bugnumber="rdar://problem/64461839")
    def test_decode_sim(self):
        """Test that we can detect an Xcode SDK that is different from the host SDK 
           from the DW_AT_LLVM_sdk attribute."""
        arch = self.getArchitecture()
        self.build(dictionary={'TRIPLE': arch+'-apple-ios-simulator', 'ARCH': arch})
        log = self.getBuildArtifact("types.log")
        self.expect("log enable lldb types -f " + log)

        lldbutil.run_to_name_breakpoint(self, 'main')

        self.expect("expression 1")
        self.check_log(log, "iPhoneSimulator")

    @swiftTest
    @skipUnlessDarwin
    @skipIfDarwinEmbedded
    def test_override(self):
        """Test that we can override the Xcode SDK using the target setting."""
        self.build()
        log = self.getBuildArtifact("types.log")
        self.expect("log enable lldb types -f " + log)
        ios_sdk = subprocess.check_output(
            ['xcrun', '--show-sdk-path', '--sdk', 'iphonesimulator']
            ).decode("utf-8").strip()
        self.assertGreater(len(ios_sdk), len('iphonesimulator'),
                           "couldn't find an iOS SDK")
        self.expect("settings set target.sdk-path " + str(ios_sdk))

        lldbutil.run_to_name_breakpoint(self, 'main')

        self.expect("expression 1")
        self.check_log(log, ios_sdk)



"""
Test that lldb persistent variables works correctly.
"""


import lldb
from lldbsuite.test.lldbtest import *


class PersistentVariablesTestCase(TestBase):
    def test_persistent_variables(self):
        """Test that lldb persistent variables works correctly."""
        self.build()
        lldbutil.run_to_source_breakpoint(
            self, "// break here", lldb.SBFileSpec("main.c")
        )

        self.runCmd("expr int $i = i")
        self.expect_expr("$i == i", result_type="bool", result_value="true")
        self.expect_expr("$i + 1", result_type="int", result_value="6")
        self.expect_expr("$i + 3", result_type="int", result_value="8")
        self.expect_expr("$1 + $2", result_type="int", result_value="14")
        self.expect_expr("$3", result_type="int", result_value="14")
        self.expect_expr("$2", result_type="int", result_value="8")
        self.expect_expr("(int)-2", result_type="int", result_value="-2")
        self.expect_expr("$4", result_type="int", result_value="-2")
        self.expect_expr("$4 > (int)31", result_type="bool", result_value="false")
        self.expect_expr("(long)$4", result_type="long", result_value="-2")

        # Try assigning an existing persistent veriable with a numeric name.
        self.expect(
            "expr int $2 = 1234",
            error=True,
            substrs=[
                "Error [IRForTarget]: Names starting with $0, $1, ... are reserved for use as result names"
            ],
        )
        # $2 should still have its original value.
        self.expect_expr("$2", result_type="int", result_value="8")

        # Try assigning an non-existing persistent veriable with a numeric name.
        self.expect(
            "expr int $200 = 3",
            error=True,
            substrs=[
                "Error [IRForTarget]: Names starting with $0, $1, ... are reserved for use as result names"
            ],
        )
        # Test that $200 wasn't created by the previous expression.
        self.expect(
            "expr $200", error=True, substrs=["use of undeclared identifier '$200'"]
        )

        # Try redeclaring the persistent variable with the same type.
        # This should be rejected as we treat them as if they are globals.
        self.expect(
            "expr int $i = 123",
            error=True,
            substrs=["error: redefinition of persistent variable '$i'"],
        )
        self.expect_expr("$i", result_type="int", result_value="5")

        # Try redeclaring the persistent variable with another type. Should
        # also be rejected.
        self.expect(
            "expr long $i = 123",
            error=True,
            substrs=["error: redefinition of persistent variable '$i'"],
        )
        self.expect_expr("$i", result_type="int", result_value="5")

        # Try assigning the persistent variable a new value.
        self.expect("expr $i = 55")
        self.expect_expr("$i", result_type="int", result_value="55")




"""
Test that a C++ class is visible in Swift.
"""
from lldbsuite.test.lldbtest import *
from lldbsuite.test.decorators import *


class TestSwiftForwardInteropVariadicTemplateTypes(TestBase):

    @swiftTest
    def test(self):
        self.build()
        
        _, _, _, _= lldbutil.run_to_source_breakpoint(
            self, 'Set breakpoint here', lldb.SBFileSpec('main.swift'))

        self.expect('v pair', substrs=['Pair', 'Tuple<OtherCxxClass>', '_t', 
            'v = false', '_t', 'a1', '10', 'a2', '20', 'a3', '30'])
        self.expect('expr pair', substrs=['Pair', 'Tuple<OtherCxxClass>', '_t', 
            'v = false', '_t', 'a1', '10', 'a2', '20', 'a3', '30'])

        # rdar://106459037 (Swift/C++ interop: Variadic templates aren't displayed correctly)
        # self.expect('v variadic', substrs=['Tuple<CxxClass, OtherCxxClass>', '_t', 
        #    'v = false', 'a1', '10', 'a2', '20', 'a3', '30']) 



import lldb
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil


class TestCppIncompleteTypes(TestBase):
    @skipIf(compiler="gcc")
    def test_limit_debug_info(self):
        self.build()
        frame = self.get_test_frame("limit")

        value_f = frame.EvaluateExpression("f")
        self.assertTrue(value_f.IsValid(), "'expr f' results in a valid SBValue object")
        self.assertSuccess(value_f.GetError(), "'expr f' is successful")

        value_a = frame.EvaluateExpression("a")
        self.assertTrue(value_a.IsValid(), "'expr a' results in a valid SBValue object")
        self.assertSuccess(value_a.GetError(), "'expr a' is successful")

    @skipIf(compiler="gcc")
    # Clang on Windows asserts in external record layout in this case.
    @skipIfWindows
    def test_partial_limit_debug_info(self):
        self.build()
        frame = self.get_test_frame("nolimit")

        value_f = frame.EvaluateExpression("f")
        self.assertTrue(value_f.IsValid(), "'expr f' results in a valid SBValue object")
        self.assertSuccess(value_f.GetError(), "'expr f' is successful")

        value_a = frame.EvaluateExpression("a")
        self.assertTrue(value_a.IsValid(), "'expr a' results in a valid SBValue object")
        self.assertSuccess(value_a.GetError(), "'expr a' is successful")

    def get_test_frame(self, exe):
        # Get main source file
        src_file = "main.cpp"
        src_file_spec = lldb.SBFileSpec(src_file)

        (target, process, thread, main_breakpoint) = lldbutil.run_to_source_breakpoint(
            self, "break here", src_file_spec, exe_name=exe
        )
        # Get frame for current thread
        return thread.GetSelectedFrame()



import lldb
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil
from lldbsuite.test.decorators import *


class TestCase(TestBase):
    def test_invalid_arg(self):
        self.build()

        lldbutil.run_to_source_breakpoint(
            self, "// break here", lldb.SBFileSpec("main.cpp")
        )

        self.expect(
            "thread select -1", error=True, startstr="error: Invalid thread index '-1'"
        )
        self.expect(
            "thread select 0x1ffffffff",
            error=True,
            startstr="error: Invalid thread index '0x1ffffffff'",
        )
        # Parses but not a valid thread id.
        self.expect(
            "thread select 0xffffffff",
            error=True,
            startstr="error: invalid thread #0xffffffff.",
        )



# TestSwiftReferenceStorageTypes.py
#
# This source file is part of the Swift.org open source project
#
# Copyright (c) 2014 - 2016 Apple Inc. and the Swift project authors
# Licensed under Apache License v2.0 with Runtime Library Exception
#
# See https://swift.org/LICENSE.txt for license information
# See https://swift.org/CONTRIBUTORS.txt for the list of Swift project authors
#
# ------------------------------------------------------------------------------
"""
Test weak, unowned and unmanaged types
"""
import lldb
from lldbsuite.test.lldbtest import *
from lldbsuite.test.decorators import *
import lldbsuite.test.lldbutil as lldbutil
import os
import unittest2


class TestSwiftReferenceStorageTypes(TestBase):
    @decorators.skipIf(archs=['ppc64le']) #SR-10215
    @swiftTest
    @skipIf(oslist=["linux"], bugnumber="rdar://76592966")
    def test_swift_reference_storage_types(self):
        """Test weak, unowned and unmanaged types"""
        self.build()
        lldbutil.run_to_source_breakpoint(
            self, 'Set breakpoint here', lldb.SBFileSpec('main.swift'))

        self.expect('frame variable myclass.sub_001', substrs=['x = 1'])
        self.expect('frame variable myclass.sub_002', substrs=['x = 1'])
        self.expect('frame variable myclass.sub_003', substrs=['x = 1'])
        self.expect('frame variable myclass.sub_004', substrs=['x = 1'])
        self.expect('frame variable myclass.sub_005', substrs=['x = 1'])
        self.expect('frame variable myclass.sub_006', substrs=['x = 1'])

        self.expect('expression myclass.sub_001', substrs=['x = 1'])
        self.expect('expression myclass.sub_002', substrs=['x = 1'])
        self.expect('expression myclass.sub_003', substrs=['x = 1'])
        self.expect('expression myclass.sub_004', substrs=['x = 1'])
        self.expect('expression myclass.sub_005', substrs=['x = 1'])
        self.expect('expression myclass.sub_006', substrs=['x = 1'])

        self.expect('expression myclass.sub_001!', substrs=['x = 1'])
        self.expect('expression myclass.sub_002!', substrs=['x = 1'])
        self.expect('expression myclass.sub_003!', substrs=['x = 1'])
        self.expect('expression myclass.sub_004!', substrs=['x = 1'])

        myclass = self.frame().FindVariable("myclass")
        sub_001 = myclass.GetChildMemberWithName("sub_001")
        sub_002 = myclass.GetChildMemberWithName("sub_002")
        sub_003 = myclass.GetChildMemberWithName("sub_003")
        sub_004 = myclass.GetChildMemberWithName("sub_004")
        sub_005 = myclass.GetChildMemberWithName("sub_005")
        sub_006 = myclass.GetChildMemberWithName("sub_006")

        sub_001_type = sub_001.GetType()
        sub_002_type = sub_002.GetType()
        sub_003_type = sub_003.GetType()
        sub_004_type = sub_004.GetType()
        sub_005_type = sub_005.GetType()
        sub_006_type = sub_006.GetType()

        self.assertTrue(sub_001_type.IsValid(), "001.GetType() is valid")
        self.assertTrue(sub_002_type.IsValid(), "002.GetType() is valid")
        self.assertTrue(sub_003_type.IsValid(), "003.GetType() is valid")
        self.assertTrue(sub_004_type.IsValid(), "004.GetType() is valid")
        self.assertTrue(sub_005_type.IsValid(), "005.GetType() is valid")
        self.assertTrue(sub_006_type.IsValid(), "006.GetType() is valid")

        sub_001_type.GetTypeClass()
        sub_002_type.GetTypeClass()
        sub_003_type.GetTypeClass()
        sub_004_type.GetTypeClass()
        sub_005_type.GetTypeClass()
        sub_006_type.GetTypeClass()
